from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme

img_dir = Path(__file__).parent / "images"


def marriage(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").resize_height(1080)
    img_w, img_h = img.size
    if img_w > 1500:
        img_w = 1500
    elif img_w < 800:
        img_h = int(img_h * img_w / 800)
    frame = img.resize_canvas((img_w, img_h)).resize_height(1080)
    left = BuildImage.open(img_dir / "0.png")
    right = BuildImage.open(img_dir / "1.png")
    frame.paste(left, alpha=True).paste(
        right, (frame.width - right.width, 0), alpha=True
    )
    return frame.save_jpg()


add_meme(
    "marriage",
    marriage,
    min_images=1,
    max_images=1,
    keywords=["结婚申请", "结婚登记"],
    date_created=datetime(2022, 5, 31),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime

import numpy as np
from PIL import Image, ImageDraw
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif


def crop_to_triangle(img: BuildImage) -> BuildImage:
    img_w, img_h = img.size
    frame = Image.new("RGBA", img.size, (0, 0, 0, 0))
    mask = Image.new("L", img.size, 0)
    draw = ImageDraw.Draw(mask)
    draw.polygon([(img_w // 2, 0), (0, img_h), (img_w, img_h)], fill=255)
    frame.paste(img.image, (0, 0), mask)
    return BuildImage(frame)


def pyramid(images: list[BuildImage], texts, args):
    img_w = 300
    img_h = 300
    fov = 45 * np.pi / 180
    z = np.sqrt(img_w**2 + img_h**2) / 2 / np.tan(fov / 2)
    a = 180
    h = 180
    rh = int(np.sqrt(a**2 + h**2))

    def rotate_y(img: BuildImage, theta: float) -> tuple[tuple[int, int], BuildImage]:
        mat = np.array(
            [
                [np.cos(theta), 0, np.sin(theta)],
                [0, 1, 0],
                [-np.sin(theta), 0, np.cos(theta)],
            ]
        )
        orgs = [
            np.array([-a / 2, -h / 2, 0]),
            np.array([a / 2, -h / 2, 0]),
            np.array([a / 2, h / 2, a / 2]),
            np.array([-a / 2, h / 2, a / 2]),
        ]
        dsts = []
        for i in range(4):
            dst = mat.dot(orgs[i])
            dsts.append(
                [
                    int(dst[0] * z / (z - dst[2])),
                    int(dst[1] * z / (z - dst[2])),
                ]
            )
        min_x = min(dsts, key=lambda x: x[0])[0]
        min_y = min(dsts, key=lambda x: x[1])[1]
        for i in range(4):
            dsts[i][0] -= min_x
            dsts[i][1] -= min_y
        pos = (img_w // 2 + min_x, img_h // 2 + min_y)
        img = img.convert("RGBA").resize((a, rh), keep_ratio=True)
        img = crop_to_triangle(img)
        img = img.perspective(tuple(dsts))
        return pos, img

    if len(images) == 1:
        images.append(images[0])
    frame_num_per_image = 15
    frame_num = len(images) * frame_num_per_image
    theta_step = 90 / frame_num_per_image

    def maker(i: int) -> Maker:
        img_idx1 = i // frame_num_per_image
        img_idx2 = (img_idx1 + 1) % len(images)
        theta1 = i % frame_num_per_image * theta_step
        theta2 = theta1 - 90

        def make(imgs: list[BuildImage]) -> BuildImage:
            frame = BuildImage.new("RGBA", (img_w, img_h))
            pos1, img1 = rotate_y(imgs[img_idx1], theta1 / 180 * np.pi)
            pos2, img2 = rotate_y(imgs[img_idx2], theta2 / 180 * np.pi)
            frame.paste(img2, pos2, alpha=True)
            frame.paste(img1, pos1, alpha=True)
            return frame

        return make

    return make_gif_or_combined_gif(
        images, maker, frame_num, 0.06, FrameAlignPolicy.extend_loop
    )


add_meme(
    "pyramid",
    pyramid,
    min_images=1,
    max_images=4,
    keywords=["四棱锥", "金字塔"],
    date_created=datetime(2024, 8, 16),
    date_modified=datetime(2024, 8, 18),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif

img_dir = Path(__file__).parent / "images"


def clauvio_twist(images: list[BuildImage], texts, args):
    params = [
        (0, (45, 144)),  # 0
        (0, (45, 144)),  # 1
        (0, (45, 144)),  # 2
        (2, (45, 141)),  # 3
        (4, (45, 141)),  # 4
        (8, (40, 141)),  # 5
        (12, (38, 142)),  # 6
        (30, (32, 148)),  # 7
        (75, (25, 158)),  # 8
        (115, (0, 160)),  # 9
        (130, (0, 160)),  # 10
        (125, (0, 155)),  # 11
        (120, (0, 150)),  # 12
        (115, (0, 148)),  # 13
        (110, (5, 146)),  # 14
        (85, (14, 146)),  # 15
        (70, (19, 146)),  # 16
        (45, (28, 144)),  # 17
        (37, (38, 141)),  # 18
        (10, (42, 144)),  # 19
    ]

    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            img = imgs[0].convert("RGBA").resize((100, 100), keep_ratio=True).circle()
            angle, pos = params[i % 20]
            bg = BuildImage.open(img_dir / f"{i}.png")
            return bg.paste(img.rotate(angle), pos, alpha=True)

        return make

    return make_gif_or_combined_gif(
        images, maker, 40, 0.05, FrameAlignPolicy.extend_loop
    )


add_meme(
    "clauvio_twist",
    clauvio_twist,
    min_images=1,
    max_images=1,
    keywords=["鼠鼠搓"],
    date_created=datetime(2024, 8, 31),
    date_modified=datetime(2024, 8, 31),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def worship(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA")
    points = ((0, -30), (135, 17), (135, 145), (0, 140))
    paint = img.square().resize((150, 150)).perspective(points)
    frames: list[IMG] = []
    for i in range(10):
        frame = BuildImage.open(img_dir / f"{i}.png")
        frame.paste(paint, below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.04)


add_meme(
    "worship",
    worship,
    min_images=1,
    max_images=1,
    keywords=["膜", "膜拜"],
    date_created=datetime(2022, 2, 10),
    date_modified=datetime(2023, 2, 14),
)



from dataclasses import dataclass, field
from datetime import datetime
from io import BytesIO
from pathlib import Path
from typing import Any, Callable, Literal, Optional, TypeVar, Union

from arclet.alconna import ArgFlag, Args, Empty, Option
from arclet.alconna.action import Action
from pil_utils import BuildImage
from pydantic import BaseModel, ValidationError

from .compat import type_validate_python
from .exception import (
    ArgModelMismatch,
    ImageNumberMismatch,
    OpenImageFailed,
    TextNumberMismatch,
    TextOrNameNotEnough,
)
from .utils import random_image, random_text


class UserInfo(BaseModel):
    name: str = ""
    gender: Literal["male", "female", "unknown"] = "unknown"


class MemeArgsModel(BaseModel):
    user_infos: list[UserInfo] = []


ArgsModel = TypeVar("ArgsModel", bound=MemeArgsModel)

MemeFunction = Callable[[list[BuildImage], list[str], ArgsModel], BytesIO]


class ParserArg(BaseModel):
    name: str
    value: str
    default: Optional[Any] = None
    flags: Optional[list[ArgFlag]] = None


class ParserOption(BaseModel):
    names: list[str]
    args: Optional[list[ParserArg]] = None
    dest: Optional[str] = None
    default: Optional[Any] = None
    action: Optional[Action] = None
    help_text: Optional[str] = None
    compact: bool = False

    def option(self) -> Option:
        args = Args()
        for arg in self.args or []:
            args.add(
                name=arg.name,
                value=arg.value,
                default=arg.default or Empty,
                flags=arg.flags,
            )

        return Option(
            name="|".join(self.names),
            args=args,
            dest=self.dest,
            default=self.default or Empty,
            action=self.action,
            help_text=self.help_text,
            compact=self.compact,
        )


class CommandShortcut(BaseModel):
    key: str
    args: Optional[list[str]] = None
    humanized: Optional[str] = None


@dataclass
class MemeArgsType:
    args_model: type[MemeArgsModel]
    args_examples: list[MemeArgsModel] = field(default_factory=list)
    parser_options: list[ParserOption] = field(default_factory=list)


@dataclass
class MemeParamsType:
    min_images: int = 0
    max_images: int = 0
    min_texts: int = 0
    max_texts: int = 0
    default_texts: list[str] = field(default_factory=list)
    args_type: Optional[MemeArgsType] = None


@dataclass
class Meme:
    key: str
    function: MemeFunction
    params_type: MemeParamsType
    keywords: list[str] = field(default_factory=list)
    shortcuts: list[CommandShortcut] = field(default_factory=list)
    tags: set[str] = field(default_factory=set)
    date_created: datetime = datetime(2021, 5, 4)
    date_modified: datetime = datetime.now()

    def __call__(
        self,
        *,
        images: Union[list[str], list[Path], list[bytes], list[BytesIO]] = [],
        texts: list[str] = [],
        args: dict[str, Any] = {},
    ) -> BytesIO:
        if not (
            self.params_type.min_images <= len(images) <= self.params_type.max_images
        ):
            raise ImageNumberMismatch(
                self.params_type.min_images, self.params_type.max_images
            )

        if not (self.params_type.min_texts <= len(texts) <= self.params_type.max_texts):
            raise TextNumberMismatch(
                self.params_type.min_texts, self.params_type.max_texts
            )

        if args_type := self.params_type.args_type:
            args_model = args_type.args_model
        else:
            args_model = MemeArgsModel

        try:
            model = type_validate_python(args_model, args)
        except ValidationError as e:
            raise ArgModelMismatch(str(e))

        imgs: list[BuildImage] = []
        try:
            for image in images:
                if isinstance(image, bytes):
                    image = BytesIO(image)
                imgs.append(BuildImage.open(image))
        except Exception as e:
            raise OpenImageFailed(str(e))

        return self.function(imgs, texts, model)

    def generate_preview(self, *, args: dict[str, Any] = {}) -> BytesIO:
        default_images = [random_image() for _ in range(self.params_type.min_images)]
        default_texts = (
            self.params_type.default_texts.copy()
            if (
                self.params_type.min_texts
                <= len(self.params_type.default_texts)
                <= self.params_type.max_texts
            )
            else [random_text() for _ in range(self.params_type.min_texts)]
        )

        def _generate_preview(images: list[bytes], texts: list[str]):
            try:
                return self.__call__(images=images, texts=texts, args=args)
            except TextOrNameNotEnough:
                texts.append(random_text())
                return _generate_preview(images, texts)

        return _generate_preview(default_images, default_texts)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def dog_of_vtb(images: list[BuildImage], texts, args):
    frame = BuildImage.open(img_dir / "0.png")

    def make(imgs: list[BuildImage]) -> BuildImage:
        points = ((0, 0), (579, 0), (584, 430), (5, 440))
        img = imgs[0].convert("RGBA").resize((600, 450), keep_ratio=True)
        return frame.copy().paste(img.perspective(points), (97, 32), below=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "dog_of_vtb",
    dog_of_vtb,
    min_images=1,
    max_images=1,
    keywords=["管人痴"],
    date_created=datetime(2023, 4, 18),
    date_modified=datetime(2023, 4, 18),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def capoo_rip(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").resize((150, 100), keep_ratio=True)
    img_left = img.crop((0, 0, 75, 100))
    img_right = img.crop((75, 0, 150, 100))
    params1 = [
        [(61, 196), ((140, 68), (0, 59), (33, 0), (165, 8))],
        [(63, 196), ((136, 68), (0, 59), (29, 0), (158, 13))],
        [(62, 195), ((137, 72), (0, 58), (27, 0), (167, 11))],
        [(95, 152), ((0, 8), (155, 0), (163, 107), (13, 112))],
        [(108, 129), ((0, 6), (128, 0), (136, 113), (10, 117))],
        [(84, 160), ((0, 6), (184, 0), (190, 90), (10, 97))],
    ]
    params2 = [
        (
            [(78, 158), ((0, 3), (86, 0), (97, 106), (16, 106))],
            [(195, 156), ((0, 4), (82, 0), (85, 106), (15, 110))],
        ),
        (
            [(89, 156), ((0, 0), (80, 0), (94, 100), (14, 100))],
            [(192, 151), ((0, 7), (79, 3), (82, 107), (11, 112))],
        ),
    ]
    raw_frames = [BuildImage.open(img_dir / f"{i}.png") for i in range(8)]
    for i in range(6):
        pos, points = params1[i]
        raw_frames[i].paste(img.perspective(points), pos, below=True)
    for i in range(2):
        (pos1, points1), (pos2, points2) = params2[i]
        raw_frames[i + 6].paste(img_left.perspective(points1), pos1, below=True)
        raw_frames[i + 6].paste(img_right.perspective(points2), pos2, below=True)

    new_frames: list[BuildImage] = []
    for i in range(3):
        new_frames += raw_frames[0:3]
    new_frames += raw_frames[3:]
    new_frames.append(raw_frames[-1])

    frames = [frame.image for frame in new_frames]
    return save_gif(frames, 0.1)


add_meme(
    "capoo_rip",
    capoo_rip,
    min_images=1,
    max_images=1,
    keywords=["咖波撕"],
    tags=MemeTags.capoo,
    date_created=datetime(2023, 4, 17),
    date_modified=datetime(2023, 4, 28),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def addiction(images: list[BuildImage], texts: list[str], args):
    frame = BuildImage.open(img_dir / "0.png")

    if texts:
        text = texts[0]
        frame = frame.resize_canvas((246, 286), direction="north", bg_color="white")
        try:
            frame.draw_text((10, 246, 236, 286), texts[0], max_fontsize=45)
        except ValueError:
            raise TextOverLength(text)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((91, 91), keep_ratio=True)
        return frame.copy().paste(img, alpha=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "addiction",
    addiction,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    keywords=["上瘾", "毒瘾发作"],
    date_created=datetime(2022, 8, 17),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif

img_dir = Path(__file__).parent / "images"


def run_away(images: list[BuildImage], texts, args):
    miku_w, miku_h = BuildImage.open(img_dir / "0.png").size
    img_w, img_h = images[0].size
    ratio = 1.2
    if img_w > img_h:
        frame_h = round(miku_h * ratio)
        frame_w = round(frame_h * img_w / img_h)
    else:
        frame_w = round(miku_w * ratio)
        frame_h = round(frame_w * img_h / img_w)

    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            frame = imgs[0].convert("RGBA").resize((frame_w, frame_h), keep_ratio=True)
            miku = BuildImage.open(img_dir / f"{i}.png")
            frame.paste(miku, (frame_w - miku_w, frame_h - miku_h), alpha=True)
            return frame

        return make

    return make_gif_or_combined_gif(
        images, maker, 42, 0.03, FrameAlignPolicy.extend_loop
    )


add_meme(
    "run_away",
    run_away,
    min_images=1,
    max_images=1,
    keywords=["快逃"],
    tags=MemeTags.miku,
    date_created=datetime(2024, 7, 23),
    date_modified=datetime(2024, 7, 23),
)



import math
from datetime import datetime

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif


def swirl_turn(images: list[BuildImage], texts, args):
    frame_num = 40

    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            img = imgs[0].convert("RGBA").circle().resize((100, 100))
            start_angle = i * 360 / frame_num
            frame = BuildImage.new("RGBA", (300, 300))
            num = 24
            for j in range(num):
                angle = start_angle + j * 360 / num
                x = 150 + 75 * math.cos(math.radians(angle))
                y = 150 + 75 * math.sin(math.radians(angle))
                frame.paste(img, (round(x - 50), round(y - 50)), alpha=True)
            return frame

        return make

    return make_gif_or_combined_gif(
        images, maker, frame_num, 0.02, FrameAlignPolicy.extend_loop
    )


add_meme(
    "swirl_turn",
    swirl_turn,
    min_images=1,
    max_images=1,
    keywords=["回旋转", "旋风转"],
    date_created=datetime(2024, 5, 7),
    date_modified=datetime(2024, 5, 7),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def wallpaper(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").resize((515, 383), keep_ratio=True)
    frames: list[IMG] = []
    for i in range(8):
        frames.append(BuildImage.open(img_dir / f"{i}.png").image)
    for i in range(8, 20):
        frame = BuildImage.open(img_dir / f"{i}.png")
        frame.paste(img, (176, -9), below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.07)


add_meme(
    "wallpaper",
    wallpaper,
    min_images=1,
    max_images=1,
    keywords=["墙纸"],
    tags=MemeTags.rick,
    date_created=datetime(2022, 3, 9),
    date_modified=datetime(2023, 2, 14),
)



import asyncio
import hashlib
import json
import time
from pathlib import Path

import httpx
from rich.progress import Progress

from .config import meme_config
from .log import logger
from .version import __version__


def _resource_url(base_url: str, name: str) -> str:
    return f"{base_url}v{__version__}/{name}"


# https://github.com/mnixry/nonebot-plugin-gocqhttp/blob/main/nonebot_plugin_gocqhttp/process/download.py
async def get_fastest_mirror() -> list[str]:
    assert meme_config.resource.resource_urls, "No resource url specified."

    async def head_mirror(client: httpx.AsyncClient, base_url: str):
        begin_time = time.time()
        response = await client.head(
            _resource_url(base_url, "resources/fonts/NotoSansSC-Regular.ttf"), timeout=5
        )
        response.raise_for_status()
        elapsed_time = (time.time() - begin_time) * 1000
        return {"base_url": base_url, "elapsed_time": elapsed_time}

    async with httpx.AsyncClient() as client:
        results = await asyncio.gather(
            *(
                head_mirror(client, domain)
                for domain in meme_config.resource.resource_urls
            ),
            return_exceptions=True,
        )
    results = sorted(
        (result for result in results if not isinstance(result, BaseException)),
        key=lambda r: r["elapsed_time"],
    )
    return [result["base_url"] for result in results]


async def check_resources():
    semaphore = asyncio.Semaphore(10)

    available_urls = (
        [meme_config.resource.resource_url]
        if meme_config.resource.resource_url
        else (await get_fastest_mirror())
    )
    logger.debug(f"Available resource urls: {available_urls}")
    if not available_urls:
        logger.warning("No resource url available.")
        return

    async def _download(client: httpx.AsyncClient, name: str):
        async with semaphore:
            for base_url in available_urls:
                url = _resource_url(base_url, name)
                try:
                    resp = await client.get(url, timeout=20, follow_redirects=True)
                    resp.raise_for_status()
                    return resp.content
                except httpx.HTTPError:
                    pass
            logger.warning(f"{name} download failed！")

    async with httpx.AsyncClient() as client:
        if content := await _download(client, "resources/resource_list.json"):
            resource_list = json.loads(content.decode("utf-8"))
        else:
            return

    download_list: list[tuple[Path, str]] = []
    for resource in resource_list:
        file_name = str(resource["path"])
        file_hash = str(resource["hash"])
        file_path = Path(__file__).parent / "memes" / file_name
        if (
            file_path.exists()
            and hashlib.md5(file_path.read_bytes()).hexdigest() == file_hash
        ):
            continue
        else:
            download_list.append((file_path, f"meme_generator/memes/{file_name}"))

    if len(download_list):
        logger.info("Downloading images ...")
    else:
        return

    async with httpx.AsyncClient() as client:

        async def download_image(file_path: Path, file_name: str):
            if content := await _download(client, file_name):
                file_path.parent.mkdir(parents=True, exist_ok=True)
                with file_path.open("wb") as f:
                    f.write(content)

        with Progress(
            *Progress.get_default_columns(), "[yellow]{task.completed}/{task.total}"
        ) as progress:
            progress_task = progress.add_task(
                "[green]Downloading...", total=len(download_list)
            )
            tasks = [
                download_image(file_path, file_name)
                for file_path, file_name in download_list
            ]
            for task in asyncio.as_completed(tasks):
                await task
                progress.update(progress_task, advance=1)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def washer(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA")
    frame = BuildImage.open(img_dir / "0.png")
    frames: list[IMG] = []
    for i in range(0, 360, 30):
        frames.append(
            frame.copy()
            .paste(img.rotate(-i).resize((74, 74)), (63, 56), below=True)
            .image
        )
    return save_gif(frames, 0.1)


add_meme(
    "washer",
    washer,
    min_images=1,
    max_images=1,
    keywords=["洗衣机"],
    date_created=datetime(2024, 1, 18),
    date_modified=datetime(2024, 1, 18),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def top_notch(images, texts: list[str], args):
    frame = BuildImage.open(img_dir / "0.png")
    text = f"好，不愧是顶尖{texts[0]}"
    try:
        frame.draw_text(
            (10, 240, 420, 300),
            text,
            allow_wrap=True,
            max_fontsize=40,
            min_fontsize=20,
        )
    except ValueError:
        raise TextOverLength(texts[0])
    return frame.save_png()


add_meme(
    "top_notch",
    top_notch,
    min_texts=1,
    max_texts=1,
    default_texts=["运营"],
    keywords=["顶尖"],
    date_created=datetime(2024, 8, 17),
    date_modified=datetime(2024, 8, 17),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def out(images: list[BuildImage], texts, args):
    frame = BuildImage.open(img_dir / "out.png")

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA")
        out = frame.copy().resize_width(int(img.width * 0.5), keep_ratio=True)
        return img.paste(
            out,
            (
                min(img.width - out.width - 10, int(img.width * 0.4)),
                min(img.height - out.height - 10, int(img.height * 0.7)),
            ),
            alpha=True,
        )

    return make_jpg_or_gif(images, make)


add_meme(
    "out",
    out,
    min_images=1,
    max_images=1,
    keywords=["out"],
    date_created=datetime(2024, 4, 26),
    date_modified=datetime(2024, 4, 26),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def bite(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square()
    frames: list[IMG] = []
    # fmt: off
    locs = [
        (90, 90, 105, 150), (90, 83, 96, 172), (90, 90, 106, 148),
        (88, 88, 97, 167), (90, 85, 89, 179), (90, 90, 106, 151)
    ]
    # fmt: on
    for i in range(6):
        frame = BuildImage.open(img_dir / f"{i}.png")
        w, h, x, y = locs[i]
        frame.paste(img.resize((w, h)), (x, y), below=True)
        frames.append(frame.image)
    for i in range(6, 16):
        frame = BuildImage.open(img_dir / f"{i}.png")
        frames.append(frame.image)
    return save_gif(frames, 0.07)


add_meme(
    "bite",
    bite,
    min_images=1,
    max_images=1,
    keywords=["啃"],
    date_created=datetime(2022, 2, 15),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.tags import MemeTags
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def capoo_say_one_loop(text: str) -> list[IMG]:
    text_frame = BuildImage.new("RGBA", (80, 80))
    try:
        text_frame.draw_text(
            (0, 0, 80, 80),
            text,
            max_fontsize=80,
            min_fontsize=20,
            allow_wrap=True,
            fontname="FZKaTong-M19S",
            lines_align="center",
        )
    except ValueError:
        raise TextOverLength(text)

    params = [
        None,
        None,
        None,
        (45, 45, 74, 112, 25),
        (73, 73, 41, 42, 17),
        (80, 80, 43, 36, 0),
        (80, 80, 43, 30, 0),
        (78, 78, 44, 30, 0),
        (78, 78, 44, 29, 0),
        None,
    ]

    frames: list[IMG] = []
    for i in range(10):
        frame = BuildImage.open(img_dir / f"{i}.png")
        param = params[i]
        if param:
            x, y, w, h, angle = param
            frame.paste(
                text_frame.resize((x, y)).rotate(angle, expand=True), (w, h), alpha=True
            )
        frames.append(frame.image)
    return frames


def capoo_say(images, texts: list[str], args):
    frames = sum([capoo_say_one_loop(text) for text in texts], [])
    return save_gif(frames, 0.1)


add_meme(
    "capoo_say",
    capoo_say,
    min_texts=1,
    max_texts=10,
    default_texts=["寄"],
    keywords=["咖波说"],
    tags=MemeTags.capoo,
    date_created=datetime(2023, 3, 28),
    date_modified=datetime(2023, 3, 30),
)



from datetime import datetime
from pathlib import Path

from PIL.ImageEnhance import Brightness
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import make_png_or_gif

img_dir = Path(__file__).parent / "images"


def erised_mirror(images: list[BuildImage], texts: list[str], args):
    frame = BuildImage.open(img_dir / "0.png")

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((360, 207), keep_ratio=True)
        img = img.color_mask((57, 78, 125))
        img = BuildImage(Brightness(img.image).enhance(0.7))
        return frame.copy().paste(
            img.perspective(((0, 0), (360, 0), (367, 207), (7, 207))),
            (55, 578),
            below=True,
        )

    return make_png_or_gif(images, make)


add_meme(
    "erised_mirror",
    erised_mirror,
    min_images=1,
    max_images=1,
    keywords=["意若思镜"],
    tags=MemeTags.harry_potter,
    date_created=datetime(2024, 8, 31),
    date_modified=datetime(2024, 8, 31),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def jump(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").circle().resize((40, 40))
    locs = [
        (15, 50),
        (13, 43),
        (15, 23),
        (14, 4),
        (16, -3),
        (16, -4),
        (14, 4),
        (15, 31),
    ]
    frames: list[IMG] = []
    for i in range(8):
        frame = BuildImage.open(img_dir / f"{i}.png")
        frame.paste(img, locs[i], alpha=True)
        frames.append(frame.image)
    return save_gif(frames, 0.05)


add_meme(
    "jump",
    jump,
    min_images=1,
    max_images=1,
    keywords=["跳"],
    date_created=datetime(2024, 7, 14),
    date_modified=datetime(2024, 7, 14),
)



from datetime import datetime
from typing import Literal, NamedTuple

from arclet.alconna import store_value
from PIL.Image import Transpose
from pil_utils import BuildImage
from pydantic import Field

from meme_generator import (
    MemeArgsModel,
    MemeArgsType,
    ParserArg,
    ParserOption,
    add_meme,
)
from meme_generator.utils import make_jpg_or_gif

help_text = "对称方向，包含 left、right、top、bottom"


class Model(MemeArgsModel):
    direction: Literal["left", "right", "top", "bottom"] = Field(
        "left", description=help_text
    )


args_type = MemeArgsType(
    args_model=Model,
    args_examples=[
        Model(direction="left"),
        Model(direction="right"),
        Model(direction="top"),
        Model(direction="bottom"),
    ],
    parser_options=[
        ParserOption(
            names=["-d", "--direction"],
            args=[ParserArg(name="direction", value="str")],
            help_text=help_text,
        ),
        ParserOption(
            names=["--left", "左"],
            dest="direction",
            action=store_value("left"),
        ),
        ParserOption(
            names=["--right", "右"],
            dest="direction",
            action=store_value("right"),
        ),
        ParserOption(
            names=["--top", "上"],
            dest="direction",
            action=store_value("top"),
        ),
        ParserOption(
            names=["--bottom", "下"],
            dest="direction",
            action=store_value("bottom"),
        ),
    ],
)


def symmetric(images: list[BuildImage], texts, args: Model):
    img_w, img_h = images[0].size

    class Mode(NamedTuple):
        method: Transpose
        frame_size: tuple[int, int]
        size1: tuple[int, int, int, int]
        pos1: tuple[int, int]
        size2: tuple[int, int, int, int]
        pos2: tuple[int, int]

    modes: dict[str, Mode] = {
        "left": Mode(
            Transpose.FLIP_LEFT_RIGHT,
            (img_w // 2 * 2, img_h),
            (0, 0, img_w // 2, img_h),
            (0, 0),
            (img_w // 2, 0, img_w // 2 * 2, img_h),
            (img_w // 2, 0),
        ),
        "right": Mode(
            Transpose.FLIP_LEFT_RIGHT,
            (img_w // 2 * 2, img_h),
            (img_w // 2, 0, img_w // 2 * 2, img_h),
            (img_w // 2, 0),
            (0, 0, img_w // 2, img_h),
            (0, 0),
        ),
        "top": Mode(
            Transpose.FLIP_TOP_BOTTOM,
            (img_w, img_h // 2 * 2),
            (0, 0, img_w, img_h // 2),
            (0, 0),
            (0, img_h // 2, img_w, img_h // 2 * 2),
            (0, img_h // 2),
        ),
        "bottom": Mode(
            Transpose.FLIP_TOP_BOTTOM,
            (img_w, img_h // 2 * 2),
            (0, img_h // 2, img_w, img_h // 2 * 2),
            (0, img_h // 2),
            (0, 0, img_w, img_h // 2),
            (0, 0),
        ),
    }

    mode = modes[args.direction]

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0]
        first = img.convert("RGBA")
        second = img.convert("RGBA").transpose(mode.method)
        frame = BuildImage.new("RGBA", mode.frame_size)
        frame.paste(first.crop(mode.size1), mode.pos1, alpha=True)
        frame.paste(second.crop(mode.size2), mode.pos2, alpha=True)
        return frame

    return make_jpg_or_gif(images, make)


add_meme(
    "symmetric",
    symmetric,
    min_images=1,
    max_images=1,
    args_type=args_type,
    keywords=["对称"],
    date_created=datetime(2022, 3, 14),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime

from pil_utils import BuildImage, Text2Image

from meme_generator import add_meme


def google(images, texts: list[str], args):
    text = texts[0]
    text = " ".join(text.splitlines())
    colors = ["#4285f4", "#db4437", "#f4b400", "#4285f4", "#0f9d58", "#db4437"]
    t2m = Text2Image.from_text(text, 200)
    index = 0
    for char in t2m.lines[0].chars:
        char.fill = colors[index % len(colors)]
        if char.char.strip():
            index += 1
    return BuildImage(t2m.to_image(bg_color="white", padding=(50, 50))).save_jpg()


add_meme(
    "google",
    google,
    min_texts=1,
    max_texts=1,
    default_texts=["Google"],
    keywords=["google"],
    date_created=datetime(2022, 10, 30),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def let_me_in(images: list[BuildImage], texts, args):
    frame = BuildImage.open(img_dir / "0.png")

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((510, 810), keep_ratio=True)
        return frame.copy().paste(img, (320, 0), below=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "let_me_in",
    let_me_in,
    min_images=1,
    max_images=1,
    keywords=["让我进去"],
    date_created=datetime(2024, 7, 18),
    date_modified=datetime(2024, 7, 18),
)



import math
from datetime import datetime

from arclet.alconna import store_true
from pil_utils import BuildImage
from pydantic import Field

from meme_generator import MemeArgsModel, MemeArgsType, ParserOption, add_meme
from meme_generator.utils import make_jpg_or_gif

help_text = "是否将图片变为圆形"


class Model(MemeArgsModel):
    circle: bool = Field(False, description=help_text)


args_type = MemeArgsType(
    args_model=Model,
    args_examples=[Model(circle=False), Model(circle=True)],
    parser_options=[
        ParserOption(
            names=["--circle", "圆"],
            default=False,
            action=store_true,
            help_text=help_text,
        ),
    ],
)


def kaleidoscope(images: list[BuildImage], texts, args: Model):
    def make(imgs: list[BuildImage]) -> BuildImage:
        circle_num = 10
        img_per_circle = 4
        init_angle = 0
        angle_step = 360 / img_per_circle

        def radius(n):
            return n * 50 + 100

        cx = cy = radius(circle_num)

        img = imgs[0].convert("RGBA")
        frame = BuildImage.new("RGBA", (cx * 2, cy * 2), "white")
        for i in range(circle_num):
            r = radius(i)
            img_w = i * 35 + 100
            im = img.resize_width(img_w)
            if args.circle:
                im = im.circle()
            for j in range(img_per_circle):
                angle = init_angle + angle_step * j
                im_rot = im.rotate(angle - 90, expand=True)
                x = round(cx + r * math.cos(math.radians(angle)) - im_rot.width / 2)
                y = round(cy - r * math.sin(math.radians(angle)) - im_rot.height / 2)
                frame.paste(im_rot, (x, y), alpha=True)
            init_angle += angle_step / 2
        return frame

    return make_jpg_or_gif(images, make)


add_meme(
    "kaleidoscope",
    kaleidoscope,
    min_images=1,
    max_images=1,
    args_type=args_type,
    keywords=["万花筒", "万花镜"],
    date_created=datetime(2023, 1, 8),
    date_modified=datetime(2023, 2, 14),
)



import random
from datetime import datetime
from pathlib import Path
from typing import Literal

from arclet.alconna import store_value
from pil_utils import BuildImage
from pydantic import Field

from meme_generator import MemeArgsModel, MemeArgsType, ParserOption, add_meme
from meme_generator.exception import TextOverLength
from meme_generator.tags import MemeTags

img_dir = Path(__file__).parent / "images"


help_text = "yes or no"


class Model(MemeArgsModel):
    mode: Literal["yes", "no", "random"] = Field("random", description=help_text)


args_type = MemeArgsType(
    args_model=Model,
    args_examples=[Model(mode="yes"), Model(mode="no")],
    parser_options=[
        ParserOption(names=["-y", "--yes"], dest="mode", action=store_value("yes")),
        ParserOption(names=["-n", "--no"], dest="mode", action=store_value("no")),
    ],
)


def atri_pillow(images, texts: list[str], args: Model):
    text = texts[0]
    mode = args.mode
    if mode == "random":
        mode = random.choice(["yes", "no"])
    if mode == "yes":
        text_color = (255, 0, 0, 80)
    else:
        text_color = (0, 80, 255, 80)
    frame = BuildImage.open(img_dir / f"{mode}.png")
    text_img = BuildImage.new("RGBA", (300, 150))
    try:
        text_img.draw_text(
            (20, 20, 280, 130),
            text,
            max_fontsize=120,
            min_fontsize=30,
            allow_wrap=True,
            lines_align="center",
            fontname="FZShaoEr-M11S",
            fill=text_color,
        )
    except ValueError:
        raise TextOverLength(text)
    frame.alpha_composite(text_img.rotate(-4, expand=True), (302, 288))
    border = BuildImage.open(img_dir / "border.png")
    frame.paste(border, (0, 416))
    return frame.save_png()


add_meme(
    "atri_pillow",
    atri_pillow,
    min_texts=1,
    max_texts=1,
    default_texts=["ATRI"],
    args_type=args_type,
    keywords=["亚托莉枕头"],
    tags=MemeTags.atri,
    date_created=datetime(2024, 8, 12),
    date_modified=datetime(2024, 8, 15),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def listen_music(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA")
    frame = BuildImage.open(img_dir / "0.png")
    frames: list[IMG] = []
    for i in range(0, 360, 10):
        frames.append(
            frame.copy()
            .paste(img.rotate(-i).resize((215, 215)), (100, 100), below=True)
            .image
        )
    return save_gif(frames, 0.05)


add_meme(
    "listen_music",
    listen_music,
    min_images=1,
    max_images=1,
    keywords=["听音乐"],
    date_created=datetime(2022, 3, 12),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def nahida_bite(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").resize((160, 140), keep_ratio=True)
    # fmt: off
    locs = [
        (123, 356, 158, 124), (123, 356, 158, 124), (123, 355, 158, 125),
        (122, 352, 159, 128), (122, 350, 159, 130), (122, 348, 159, 132),
        (122, 345, 159, 135), (121, 343, 160, 137), (121, 342, 160, 138),
        (121, 341, 160, 139), (121, 341, 160, 139), (121, 342, 160, 138),
        (121, 344, 160, 136), (121, 346, 160, 134), (122, 349, 159, 131),
        (122, 351, 159, 129), (122, 353, 159, 127), (123, 355, 158, 125),
    ]
    # fmt: on
    frames: list[IMG] = []
    for i in range(38):
        frame = BuildImage.open(img_dir / f"{i}.png")
        x, y, w, h = locs[i % len(locs)]
        frame.paste(img.resize((w, h)), (x, y), below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.1)


add_meme(
    "nahida_bite",
    nahida_bite,
    min_images=1,
    max_images=1,
    keywords=["纳西妲啃", "草神啃"],
    tags=MemeTags.nahida,
    date_created=datetime(2023, 6, 23),
    date_modified=datetime(2024, 8, 10),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def slap(images, texts: list[str], args):
    text = texts[0]
    frame = BuildImage.open(img_dir / "0.jpg")
    try:
        frame.draw_text(
            (20, 450, 620, 630),
            text,
            allow_wrap=True,
            max_fontsize=110,
            min_fontsize=50,
        )
    except ValueError:
        raise TextOverLength(text)
    return frame.save_jpg()


add_meme(
    "slap",
    slap,
    min_texts=1,
    max_texts=1,
    keywords=["一巴掌"],
    date_created=datetime(2022, 1, 19),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme

img_dir = Path(__file__).parent / "images"


def daynight(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").resize((333, 360), keep_ratio=True)
    img_ = images[1].convert("RGBA").resize((333, 360), keep_ratio=True)
    frame = BuildImage.open(img_dir / "0.png")
    frame.paste(img, (349, 0))
    frame.paste(img_, (349, 361))
    return frame.save_jpg()


add_meme(
    "daynight",
    daynight,
    min_images=2,
    max_images=2,
    keywords=["白天黑夜", "白天晚上"],
    date_created=datetime(2023, 10, 3),
    date_modified=datetime(2023, 10, 3),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"

defalut_text = "采访大佬经验"


def interview(images: list[BuildImage], texts: list[str], args):
    if len(images) == 2:
        self_img = images[0]
        user_img = images[1]
    else:
        self_img = BuildImage.open(img_dir / "huaji.png")
        user_img = images[0]
    self_img = self_img.convert("RGBA").square().resize((124, 124))
    user_img = user_img.convert("RGBA").square().resize((124, 124))

    text = texts[0] if texts else defalut_text

    frame = BuildImage.new("RGBA", (600, 310), "white")
    microphone = BuildImage.open(img_dir / "microphone.png")
    frame.paste(microphone, (330, 103), alpha=True)
    frame.paste(self_img, (419, 40), alpha=True)
    frame.paste(user_img, (57, 40), alpha=True)
    try:
        frame.draw_text((20, 200, 580, 310), text, max_fontsize=50, min_fontsize=20)
    except ValueError:
        raise TextOverLength(text)
    return frame.save_jpg()


add_meme(
    "interview",
    interview,
    min_images=1,
    max_images=2,
    min_texts=0,
    max_texts=1,
    default_texts=[defalut_text],
    keywords=["采访"],
    date_created=datetime(2022, 3, 15),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage, Text2Image
from pydantic import Field

from meme_generator import (
    MemeArgsModel,
    MemeArgsType,
    ParserArg,
    ParserOption,
    add_meme,
)
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"

help_text = "指定名字"


class Model(MemeArgsModel):
    name: str = Field("", description=help_text)


args_type = MemeArgsType(
    args_model=Model,
    parser_options=[
        ParserOption(
            names=["-n", "--name"],
            args=[ParserArg(name="name", value="str")],
            help_text=help_text,
        ),
    ],
)


def my_friend(images: list[BuildImage], texts: list[str], args: Model):
    name = args.name or (args.user_infos[-1].name if args.user_infos else "") or "朋友"
    img = images[0].convert("RGBA").circle().resize((100, 100))

    name_img = Text2Image.from_text(name, 25, fill="#868894").to_image()
    name_w, name_h = name_img.size
    if name_w >= 600:
        raise TextOverLength(name)

    corner1 = BuildImage.open(img_dir / "corner1.png")
    corner2 = BuildImage.open(img_dir / "corner2.png")
    corner3 = BuildImage.open(img_dir / "corner3.png")
    corner4 = BuildImage.open(img_dir / "corner4.png")
    label = BuildImage.open(img_dir / "label.png")

    def make_dialog(text: str) -> BuildImage:
        text_img = Text2Image.from_text(text, 40).wrap(600).to_image()
        text_w, text_h = text_img.size
        box_w = max(text_w, name_w + 15) + 140
        box_h = max(text_h + 103, 150)
        box = BuildImage.new("RGBA", (box_w, box_h))
        box.paste(corner1, (0, 0))
        box.paste(corner2, (0, box_h - 75))
        box.paste(corner3, (text_w + 70, 0))
        box.paste(corner4, (text_w + 70, box_h - 75))
        box.paste(BuildImage.new("RGBA", (text_w, box_h - 40), "white"), (70, 20))
        box.paste(BuildImage.new("RGBA", (text_w + 88, box_h - 150), "white"), (27, 75))
        box.paste(text_img, (70, 17 + (box_h - 40 - text_h) // 2), alpha=True)

        dialog = BuildImage.new("RGBA", (box.width + 130, box.height + 60), "#eaedf4")
        dialog.paste(img, (20, 20), alpha=True)
        dialog.paste(box, (130, 60), alpha=True)
        dialog.paste(label, (160, 25))
        dialog.paste(name_img, (260, 22 + (35 - name_h) // 2), alpha=True)
        return dialog

    dialogs = [make_dialog(text) for text in texts]
    frame_w = max(dialog.width for dialog in dialogs)
    frame_h = sum(dialog.height for dialog in dialogs)
    frame = BuildImage.new("RGBA", (frame_w, frame_h), "#eaedf4")
    current_h = 0
    for dialog in dialogs:
        frame.paste(dialog, (0, current_h))
        current_h += dialog.height
    return frame.save_jpg()


add_meme(
    "my_friend",
    my_friend,
    min_images=1,
    max_images=1,
    min_texts=1,
    max_texts=10,
    default_texts=["让我康康"],
    args_type=args_type,
    keywords=["我朋友说"],
    date_created=datetime(2022, 3, 11),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def capoo_draw(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").resize((175, 120), keep_ratio=True)
    params = (
        (((27, 0), (207, 12), (179, 142), (0, 117)), (30, 16)),
        (((28, 0), (207, 13), (180, 137), (0, 117)), (34, 17)),
    )
    raw_frames = [BuildImage.open(img_dir / f"{i}.png") for i in range(6)]
    for i in range(2):
        points, pos = params[i]
        raw_frames[4 + i].paste(img.perspective(points), pos, below=True)

    frames: list[IMG] = []
    frames.append(raw_frames[0].image)
    for i in range(4):
        frames.append(raw_frames[1].image)
        frames.append(raw_frames[2].image)
    frames.append(raw_frames[3].image)
    for i in range(6):
        frames.append(raw_frames[4].image)
        frames.append(raw_frames[5].image)

    return save_gif(frames, 0.1)


add_meme(
    "capoo_draw",
    capoo_draw,
    min_images=1,
    max_images=1,
    keywords=["咖波画"],
    tags=MemeTags.capoo,
    date_created=datetime(2023, 3, 31),
    date_modified=datetime(2023, 4, 28),
)



from datetime import datetime

from pil_utils import BuildImage

from meme_generator import add_meme


def cyan(images: list[BuildImage], texts, args):
    color = (78, 114, 184)
    frame = images[0].convert("RGB").square().resize((500, 500)).color_mask(color)
    frame.draw_text(
        (400, 40, 480, 280),
        "群\n青",
        max_fontsize=80,
        weight="bold",
        fill="white",
        stroke_ratio=0.04,
        stroke_fill=color,
    ).draw_text(
        (200, 270, 480, 350),
        "YOASOBI",
        halign="right",
        max_fontsize=40,
        fill="white",
        stroke_ratio=0.06,
        stroke_fill=color,
    )
    return frame.save_jpg()


add_meme(
    "cyan",
    cyan,
    min_images=1,
    max_images=1,
    keywords=["群青"],
    date_created=datetime(2022, 3, 18),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import make_png_or_gif

img_dir = Path(__file__).parent / "images"


def konata_watch(images: list[BuildImage], texts: list[str], args):
    frame = BuildImage.open(img_dir / "0.png")

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((270, 200), keep_ratio=True)
        return frame.copy().paste(
            img.perspective(((0, 1), (275, 0), (273, 202), (2, 216))),
            (50, 188),
            below=True,
        )

    return make_png_or_gif(images, make)


add_meme(
    "konata_watch",
    konata_watch,
    min_images=1,
    max_images=1,
    keywords=["泉此方看"],
    tags=MemeTags.konata,
    date_created=datetime(2024, 8, 18),
    date_modified=datetime(2024, 8, 19),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def rub(images: list[BuildImage], texts, args):
    self_head = images[0].convert("RGBA").circle()
    user_head = images[1].convert("RGBA").circle()
    # fmt: off
    user_locs = [
        (39, 91, 75, 75), (49, 101, 75, 75), (67, 98, 75, 75),
        (55, 86, 75, 75), (61, 109, 75, 75), (65, 101, 75, 75)
    ]
    self_locs = [
        (102, 95, 70, 80, 0), (108, 60, 50, 100, 0), (97, 18, 65, 95, 0),
        (65, 5, 75, 75, -20), (95, 57, 100, 55, -70), (109, 107, 65, 75, 0)
    ]
    # fmt: on
    frames: list[IMG] = []
    for i in range(6):
        frame = BuildImage.open(img_dir / f"{i}.png")
        x, y, w, h = user_locs[i]
        frame.paste(user_head.resize((w, h)), (x, y), alpha=True)
        x, y, w, h, angle = self_locs[i]
        frame.paste(
            self_head.resize((w, h)).rotate(angle, expand=True), (x, y), alpha=True
        )
        frames.append(frame.image)
    return save_gif(frames, 0.05)


add_meme(
    "rub",
    rub,
    min_images=2,
    max_images=2,
    keywords=["贴", "贴贴", "蹭", "蹭蹭"],
    date_created=datetime(2021, 6, 11),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import CommandShortcut, MemeArgsModel, add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def fill_head(images: list[BuildImage], texts: list[str], args: MemeArgsModel):
    name = texts[0] if texts else (args.user_infos[0].name if args.user_infos else "它")
    text = f"满脑子都是{name}"
    frame = BuildImage.open(img_dir / "0.jpg")
    try:
        frame.draw_text(
            (20, 458, frame.width - 20, 550), text, max_fontsize=65, min_fontsize=30
        )
    except ValueError:
        raise TextOverLength(name)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((210, 170), keep_ratio=True, inside=True)
        return frame.copy().paste(img, (150, 2), alpha=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "fill_head",
    fill_head,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    keywords=["满脑子"],
    shortcuts=[
        CommandShortcut(
            key=r"满脑子都是(?P<name>\S+)",
            args=["{name}"],
            humanized="满脑子都是xx",
        )
    ],
    date_created=datetime(2023, 6, 3),
    date_modified=datetime(2023, 6, 3),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def klee_eat(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square().resize((83, 83))
    # fmt: off
    locs = [
        (0, 174), (0, 174), (0, 174), (0, 174), (0, 174),
        (12, 160), (19, 152), (23, 148), (26, 145), (32, 140),
        (37, 136), (42, 131), (49, 127), (70, 126), (88, 128),
        (-30, 210), (-19, 207), (-14, 200), (-10, 188), (-7, 179),
        (-3, 170), (-3, 175), (-1, 174), (0, 174), (0, 174),
        (0, 174), (0, 174), (0, 174), (0, 174), (0, 174), (0, 174)
    ]
    # fmt: on
    frames: list[IMG] = []
    for i in range(31):
        frame = BuildImage.open(img_dir / f"{i}.png")
        frame.paste(img, locs[i], below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.1)


add_meme(
    "klee_eat",
    klee_eat,
    min_images=1,
    max_images=1,
    keywords=["可莉吃"],
    tags=MemeTags.klee,
    date_created=datetime(2022, 11, 29),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def eat(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square().resize((34, 34))
    frames = []
    for i in range(3):
        frame = BuildImage.open(img_dir / f"{i}.png")
        frame.paste(img, (2, 38), below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.05)


add_meme(
    "eat",
    eat,
    min_images=1,
    max_images=1,
    keywords=["吃"],
    date_created=datetime(2022, 2, 15),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def beat_up(images: list[BuildImage], texts, args):
    self_head = images[0].convert("RGBA").circle().resize((55, 55))
    user_head = images[1].convert("RGBA").circle().resize((45, 45))
    self_locs = [(100, 43), (110, 46), (101, 40)]
    user_locs = [(99, 136), (99, 136), (89, 140)]
    frames: list[IMG] = []
    for i in range(3):
        frame = BuildImage.open(img_dir / f"{i}.png")
        frame.paste(user_head, user_locs[i], alpha=True)
        frame.paste(self_head, self_locs[i], alpha=True)
        frames.append(frame.image)

    return save_gif(frames, 0.1)


add_meme(
    "beat_up",
    beat_up,
    min_images=2,
    max_images=2,
    keywords=["揍"],
    tags=MemeTags.tom | MemeTags.jerry,
    date_created=datetime(2024, 4, 9),
    date_modified=datetime(2024, 4, 9),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags

img_dir = Path(__file__).parent / "images"


def painter(images: list[BuildImage], texts, args):
    img = (
        images[0].convert("RGBA").resize((240, 345), keep_ratio=True, direction="north")
    )
    frame = BuildImage.open(img_dir / "0.png")
    frame.paste(img, (125, 91), below=True)
    return frame.save_jpg()


add_meme(
    "painter",
    painter,
    min_images=1,
    max_images=1,
    keywords=["小画家"],
    tags=MemeTags.griseo,
    date_created=datetime(2022, 6, 4),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime

from pil_utils import BuildImage, Text2Image

from meme_generator import MemeArgsModel, add_meme
from meme_generator.exception import TextOverLength


def follow(images: list[BuildImage], texts: list[str], args: MemeArgsModel):
    img = images[0].circle().resize((200, 200))

    if texts:
        name = texts[0]
    else:
        if args.user_infos:
            user_info = args.user_infos[0]
            name = user_info.name
            if not name:
                name = "女同" if user_info.gender == "female" else "男同"
        else:
            name = "男同"

    name_img = Text2Image.from_text(name, 60).to_image()
    follow_img = Text2Image.from_text("关注了你", 60, fill="grey").to_image()
    text_width = max(name_img.width, follow_img.width)
    if text_width >= 1000:
        raise TextOverLength(name)

    frame = BuildImage.new("RGBA", (300 + text_width + 50, 300), (255, 255, 255, 0))
    frame.paste(img, (50, 50), alpha=True)
    frame.paste(name_img, (300, 135 - name_img.height), alpha=True)
    frame.paste(follow_img, (300, 145), alpha=True)
    return frame.save_jpg()


add_meme(
    "follow",
    follow,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    keywords=["关注"],
    date_created=datetime(2022, 3, 10),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def love_you(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square()
    frames: list[IMG] = []
    locs = [(68, 65, 70, 70), (63, 59, 80, 80)]
    for i in range(2):
        heart = BuildImage.open(img_dir / f"{i}.png")
        frame = BuildImage.new("RGBA", heart.size, "white")
        x, y, w, h = locs[i]
        frame.paste(img.resize((w, h)), (x, y), alpha=True).paste(heart, alpha=True)
        frames.append(frame.image)
    return save_gif(frames, 0.2)


add_meme(
    "love_you",
    love_you,
    min_images=1,
    max_images=1,
    keywords=["永远爱你"],
    date_created=datetime(2022, 3, 13),
    date_modified=datetime(2023, 2, 14),
)



import asyncio
import hashlib
import inspect
import math
import random
import time
from collections.abc import Coroutine
from dataclasses import dataclass, field
from enum import Enum
from functools import partial, wraps
from io import BytesIO
from pathlib import Path
from typing import TYPE_CHECKING, Any, Callable, Literal, TypeVar

import httpx
from PIL.Image import Image as IMG
from pil_utils import BuildImage, Text2Image
from pil_utils.types import ColorType
from typing_extensions import ParamSpec

from .config import meme_config
from .exception import MemeFeedback

if TYPE_CHECKING:
    from .meme import Meme

resources_dir = Path(__file__).parent / "resources"


P = ParamSpec("P")
R = TypeVar("R")


def run_sync(call: Callable[P, R]) -> Callable[P, Coroutine[None, None, R]]:
    """一个用于包装 sync function 为 async function 的装饰器
    参数:
        call: 被装饰的同步函数
    """

    @wraps(call)
    async def _wrapper(*args: P.args, **kwargs: P.kwargs) -> R:
        loop = asyncio.get_running_loop()
        pfunc = partial(call, *args, **kwargs)
        result = await loop.run_in_executor(None, pfunc)
        return result

    return _wrapper


def is_coroutine_callable(call: Callable[..., Any]) -> bool:
    """检查 call 是否是一个 callable 协程函数"""
    if inspect.isroutine(call):
        return inspect.iscoroutinefunction(call)
    if inspect.isclass(call):
        return False
    func_ = getattr(call, "__call__", None)
    return inspect.iscoroutinefunction(func_)


def save_gif(frames: list[IMG], duration: float) -> BytesIO:
    output = BytesIO()
    frames[0].save(
        output,
        format="GIF",
        save_all=True,
        append_images=frames[1:],
        duration=duration * 1000,
        loop=0,
        disposal=2,
        optimize=False,
    )

    # 没有超出最大大小，直接返回
    nbytes = output.getbuffer().nbytes
    if nbytes <= meme_config.gif.gif_max_size * 10**6:
        return output

    # 超出最大大小，帧数超出最大帧数时，缩减帧数
    n_frames = len(frames)
    gif_max_frames = meme_config.gif.gif_max_frames
    if n_frames > gif_max_frames:
        index = range(n_frames)
        ratio = n_frames / gif_max_frames
        index = (int(i * ratio) for i in range(gif_max_frames))
        new_duration = duration * ratio
        new_frames = [frames[i] for i in index]
        return save_gif(new_frames, new_duration)

    # 超出最大大小，帧数没有超出最大帧数时，缩小尺寸
    new_frames = [
        frame.resize((int(frame.width * 0.9), int(frame.height * 0.9)))
        for frame in frames
    ]
    return save_gif(new_frames, duration)


def get_avg_duration(image: IMG) -> float:
    if not getattr(image, "is_animated", False):
        return 0
    total_duration = 0
    n_frames = getattr(image, "n_frames", 1)
    for i in range(n_frames):
        image.seek(i)
        total_duration += image.info.get("duration", 20)
    return total_duration / n_frames / 1000


def split_gif(image: IMG) -> list[IMG]:
    frames: list[IMG] = []
    n_frames = getattr(image, "n_frames", 1)
    for i in range(n_frames):
        image.seek(i)
        frame = image.copy()
        frames.append(frame)
    image.seek(0)
    if image.info.__contains__("transparency"):
        frames[0].info["transparency"] = image.info["transparency"]
    return frames


class FrameAlignPolicy(Enum):
    """
    要叠加的gif长度大于基准gif时，是否延长基准gif长度以对齐两个gif
    """

    no_extend = 0
    """不延长"""
    extend_first = 1
    """延长第一帧"""
    extend_last = 2
    """延长最后一帧"""
    extend_loop = 3
    """以循环方式延长"""


def get_aligned_gif_indexes(
    gif_infos: list[tuple[int, float]],
    frame_num_target: int,
    duration_target: float,
    frame_align: FrameAlignPolicy = FrameAlignPolicy.no_extend,
) -> tuple[list[list[int]], list[int]]:
    """
    将gif按照目标帧数和帧间隔对齐
    :params
        * ``gif_infos``: 每个输入gif的帧数和帧间隔，帧间隔单位为秒
        * ``frame_num_target``: 目标gif的帧数
        * ``duration_target``: 目标gif的帧间隔，单位为秒
        * ``frame_align``: 要对齐的gif长度大于目标gif时，gif长度对齐方式
    :return
        * 输入gif的帧索引列表和目标gif的帧索引列表
    """

    frame_idxs_target: list[int] = list(range(frame_num_target))

    max_total_duration_input = max(
        frame_num * duration for frame_num, duration in gif_infos
    )
    total_duration_target = frame_num_target * duration_target
    if (
        diff_duration := max_total_duration_input - total_duration_target
    ) >= duration_target:
        diff_num = math.ceil(diff_duration / duration_target)

        if frame_align == FrameAlignPolicy.extend_first:
            frame_idxs_target = [0] * diff_num + frame_idxs_target

        elif frame_align == FrameAlignPolicy.extend_last:
            frame_idxs_target += [frame_num_target - 1] * diff_num

        elif frame_align == FrameAlignPolicy.extend_loop:
            frame_num_total = frame_num_target
            # 重复目标gif，直到每个gif总时长之差在1个间隔以内，或总帧数超出最大帧数
            while frame_num_total + frame_num_target <= meme_config.gif.gif_max_frames:
                frame_num_total += frame_num_target
                frame_idxs_target += list(range(frame_num_target))
                total_duration = frame_num_total * duration_target
                if all(
                    math.fabs(
                        round(total_duration / duration / frame_num)
                        * duration
                        * frame_num
                        - total_duration
                    )
                    <= duration_target
                    for frame_num, duration in gif_infos
                ):
                    break

    frame_idxs_input: list[list[int]] = []
    for frame_num, duration in gif_infos:
        frame_idx = 0
        time_start = 0
        frame_idxs: list[int] = []
        for i in range(len(frame_idxs_target)):
            while frame_idx < frame_num:
                if (
                    frame_idx * duration
                    <= i * duration_target - time_start
                    < (frame_idx + 1) * duration
                ):
                    frame_idxs.append(frame_idx)
                    break
                else:
                    frame_idx += 1
                    if frame_idx >= frame_num:
                        frame_idx = 0
                        time_start += frame_num * duration
        frame_idxs_input.append(frame_idxs)

    return frame_idxs_input, frame_idxs_target


Maker = Callable[[list[BuildImage]], BuildImage]

GifMaker = Callable[[int], Maker]


def merge_gif(imgs: list[BuildImage], func: Maker) -> BytesIO:
    """
    合并动图
    :params
      * ``imgs``: 输入图片列表
      * ``func``: 图片处理函数，输入imgs，返回处理后的图片
    """
    images = [img.image for img in imgs]
    gif_images = [image for image in images if getattr(image, "is_animated", False)]

    if len(gif_images) == 1:
        frames: list[IMG] = []
        frame_num = getattr(gif_images[0], "n_frames", 1)
        duration = get_avg_duration(gif_images[0])
        for i in range(frame_num):
            frame_images: list[IMG] = []
            for image in images:
                if getattr(image, "is_animated", False):
                    image.seek(i)
                frame_images.append(image.copy())
            frame = func([BuildImage(image) for image in frame_images])
            frames.append(frame.image)
        return save_gif(frames, duration)

    gif_infos = [
        (getattr(image, "n_frames", 1), get_avg_duration(image)) for image in gif_images
    ]
    target_duration = min(duration for _, duration in gif_infos)
    target_gif_idx = [
        i for i, (_, duration) in enumerate(gif_infos) if duration == target_duration
    ][0]
    target_frame_num = gif_infos[target_gif_idx][0]
    gif_infos.pop(target_gif_idx)
    frame_idxs, target_frame_idxs = get_aligned_gif_indexes(
        gif_infos, target_frame_num, target_duration, FrameAlignPolicy.extend_loop
    )
    frame_idxs.insert(target_gif_idx, target_frame_idxs)

    frames: list[IMG] = []
    for i in range(len(target_frame_idxs)):
        frame_images: list[IMG] = []
        gif_idx = 0
        for image in images:
            if getattr(image, "is_animated", False):
                image.seek(frame_idxs[gif_idx][i])
                gif_idx += 1
            frame_images.append(image.copy())
        frame = func([BuildImage(image) for image in frame_images])
        frames.append(frame.image)

    return save_gif(frames, target_duration)


def make_jpg_or_gif(imgs: list[BuildImage], func: Maker) -> BytesIO:
    """
    制作静图或者动图
    :params
      * ``imgs``: 输入图片列表
      * ``func``: 图片处理函数，输入imgs，返回处理后的图片
    """
    images = [img.image for img in imgs]
    if all(not getattr(image, "is_animated", False) for image in images):
        return func(imgs).save_jpg()

    return merge_gif(imgs, func)


def make_png_or_gif(imgs: list[BuildImage], func: Maker) -> BytesIO:
    """
    制作静图或者动图
    :params
      * ``imgs``: 输入图片列表
      * ``func``: 图片处理函数，输入imgs，返回处理后的图片
    """
    images = [img.image for img in imgs]
    if all(not getattr(image, "is_animated", False) for image in images):
        return func(imgs).save_png()

    return merge_gif(imgs, func)


def make_gif_or_combined_gif(
    imgs: list[BuildImage],
    maker: GifMaker,
    frame_num: int,
    duration: float,
    frame_align: FrameAlignPolicy = FrameAlignPolicy.no_extend,
) -> BytesIO:
    """
    使用静图或动图制作gif
    :params
      * ``imgs``: 输入图片列表
      * ``maker``: 图片处理函数生成，传入第几帧，返回对应的图片处理函数
      * ``frame_num``: 目标gif的帧数
      * ``duration``: 相邻帧之间的时间间隔，单位为秒
      * ``frame_align``: 要叠加的gif长度大于基准gif时，gif长度对齐方式
    """
    images = [img.image for img in imgs]
    if all(not getattr(image, "is_animated", False) for image in images):
        return save_gif([maker(i)(imgs).image for i in range(frame_num)], duration)

    gif_infos = [
        (getattr(image, "n_frames", 1), get_avg_duration(image))
        for image in images
        if getattr(image, "is_animated", False)
    ]
    frame_idxs_input, frame_idxs_target = get_aligned_gif_indexes(
        gif_infos, frame_num, duration, frame_align
    )

    frames: list[IMG] = []
    for i, idx in enumerate(frame_idxs_target):
        frame_images: list[IMG] = []
        gif_idx = 0
        for image in images:
            if getattr(image, "is_animated", False):
                image.seek(frame_idxs_input[gif_idx][i])
                gif_idx += 1
            frame_images.append(image.copy())
        frame = maker(idx)([BuildImage(image) for image in frame_images])
        frames.append(frame.image)

    return save_gif(frames, duration)


def translate(text: str, lang_from: str = "auto", lang_to: str = "zh") -> str:
    appid = meme_config.translate.baidu_trans_appid
    apikey = meme_config.translate.baidu_trans_apikey
    if not appid or not apikey:
        raise MemeFeedback(
            '"baidu_trans_appid" 或 "baidu_trans_apikey" 未设置，请检查配置文件！'
        )
    salt = str(round(time.time() * 1000))
    sign_raw = appid + text + salt + apikey
    sign = hashlib.md5(sign_raw.encode("utf8")).hexdigest()
    params = {
        "q": text,
        "from": lang_from,
        "to": lang_to,
        "appid": appid,
        "salt": salt,
        "sign": sign,
    }
    url = "https://fanyi-api.baidu.com/api/trans/vip/translate"
    resp = httpx.get(url, params=params)
    result = resp.json()
    return result["trans_result"][0]["dst"]


def random_text() -> str:
    return random.choice(
        ["刘一", "陈二", "张三", "李四", "王五", "赵六", "孙七", "周八", "吴九", "郑十"]
    )


def random_image() -> bytes:
    return random.choice(
        list((resources_dir / "images" / "emojis").glob("*.png"))
    ).read_bytes()


@dataclass
class MemeProperties:
    disabled: bool = False
    labels: list[Literal["new", "hot"]] = field(default_factory=list)


def render_meme_list(
    meme_list: list[tuple["Meme", MemeProperties]],
    *,
    text_template: str = "{keywords}",
    add_category_icon: bool = True,
) -> BytesIO:
    TEXT_COLOR_NORMAL = "#444444"
    TEXT_COLOR_DISABLED = "#d3d3d3"
    BLOCK_COLOR_1 = "#f5f5f5"
    BLOCK_COLOR_2 = "#ffffff"
    BG_COLOR = "#fdfcf8"
    FONTSIZE = 30
    BLOCK_HEIGHT = 50

    icon_dir = resources_dir / "images" / "icons"
    category_image = BuildImage.open(icon_dir / "image.png").resize((30, 30))
    category_text = BuildImage.open(icon_dir / "text.png").resize((30, 30))
    category_image_disabled = BuildImage.open(icon_dir / "image_disabled.png").resize(
        (30, 30)
    )
    category_text_disabled = BuildImage.open(icon_dir / "text_disabled.png").resize(
        (30, 30)
    )
    label_new = BuildImage.open(icon_dir / "new.png").resize((30, 30))
    label_hot = BuildImage.open(icon_dir / "hot.png").resize((30, 30))

    def meme_text(number: int, meme: "Meme") -> str:
        return text_template.format(
            index=number + 1,
            key=meme.key,
            keywords="/".join(meme.keywords),
            shortcuts="/".join(
                shortcut.humanized or shortcut.key for shortcut in meme.shortcuts
            ),
            tags="/".join(meme.tags),
        )

    def text_block(
        text: str,
        max_width: int,
        block_color: ColorType,
        properties: MemeProperties,
        category: Literal["text", "image"] = "text",
    ) -> BuildImage:
        image = BuildImage.new("RGBA", (max_width, BLOCK_HEIGHT), block_color)
        if category == "text":
            icon = category_text_disabled if properties.disabled else category_text
        else:
            icon = category_image_disabled if properties.disabled else category_image
        x = 0
        if add_category_icon:
            image.paste(icon, (x + 10, 10), alpha=True)
            x += 50
        text_color = TEXT_COLOR_DISABLED if properties.disabled else TEXT_COLOR_NORMAL
        t2m = Text2Image.from_text(text, fontsize=FONTSIZE, fill=text_color)
        t2m.draw_on_image(image.image, (x + 5, (image.height - t2m.height) // 2))
        x += t2m.width + 10
        if "new" in properties.labels:
            image.paste(label_new, (x + 5, 10), alpha=True)
            x += 35
        if "hot" in properties.labels:
            image.paste(label_hot, (x + 5, 10), alpha=True)
            x += 35
        return image

    meme_num = len(meme_list)
    cols = math.ceil(math.sqrt(meme_num / 16))
    rows = math.ceil(meme_num / cols)

    col_images: list[BuildImage] = []
    for col in range(cols):
        col_meme_list = meme_list[col * rows : (col + 1) * rows]
        max_width = max(
            Text2Image.from_text(
                meme_text(col * rows + row, meme), fontsize=FONTSIZE
            ).width
            + (50 if add_category_icon else 0)
            + 20
            + len(properties.labels) * 35
            for row, (meme, properties) in enumerate(col_meme_list)
        )
        col_image = BuildImage.new("RGBA", (max_width, rows * BLOCK_HEIGHT), BG_COLOR)
        for row, (meme, properties) in enumerate(col_meme_list):
            text = meme_text(col * rows + row, meme)
            block_color = BLOCK_COLOR_1 if (row + col) % 2 == 0 else BLOCK_COLOR_2
            category = "text" if meme.params_type.max_images == 0 else "image"
            col_image.paste(
                text_block(text, max_width, block_color, properties, category),
                (0, row * BLOCK_HEIGHT),
            )
        col_images.append(col_image)

    margin = 30
    frame = BuildImage.new(
        "RGBA",
        (
            sum(image.width for image in col_images) + margin * 2,
            rows * BLOCK_HEIGHT + margin * 2,
        ),
        BG_COLOR,
    )
    x = margin
    y = margin
    for image in col_images:
        frame.paste(image, (x, y))
        x += image.width
    return frame.save_png()



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif

img_dir = Path(__file__).parent / "images"


def play_together(images: list[BuildImage], texts, args):
    params = (
        36 * [(194, 204, 121, 108)]
        + [
            (192, 203, 125, 111),
            (182, 200, 141, 126),
            (161, 188, 178, 159),
            (129, 171, 235, 209),
            (98, 155, 290, 258),
            (98, 155, 290, 258),
            (58, 133, 361, 321),
            (45, 126, 384, 342),
            (45, 126, 384, 342),
            (35, 121, 402, 358),
            (27, 117, 415, 370),
            (17, 111, 433, 386),
            (14, 110, 439, 391),
            (14, 110, 439, 391),
            (11, 108, 444, 395),
            (10, 108, 446, 397),
        ]
        + 8 * [(7, 106, 451, 402)]
    )

    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            x, y, w, h = params[i]
            screen = imgs[0].convert("RGBA").resize((w, h), keep_ratio=True)
            frame = BuildImage.open(img_dir / f"{i}.png")
            frame.paste(screen, (x, y), below=True)
            return frame

        return make

    return make_gif_or_combined_gif(
        images, maker, 60, 0.06, FrameAlignPolicy.extend_last
    )


add_meme(
    "play_together",
    play_together,
    min_images=1,
    max_images=1,
    keywords=["一起玩"],
    tags=MemeTags.blue_archive,
    date_created=datetime(2024, 7, 26),
    date_modified=datetime(2024, 7, 26),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"

default_text = "来玩休闲游戏啊"


def play_game(images: list[BuildImage], texts: list[str], args):
    text = texts[0] if texts else default_text
    frame = BuildImage.open(img_dir / "0.png")
    try:
        frame.draw_text(
            (20, frame.height - 70, frame.width - 20, frame.height),
            text,
            max_fontsize=40,
            min_fontsize=25,
            stroke_fill="white",
            stroke_ratio=0.06,
        )
    except ValueError:
        raise TextOverLength(text)

    def make(imgs: list[BuildImage]) -> BuildImage:
        points = ((0, 5), (227, 0), (216, 150), (0, 165))
        screen = (
            imgs[0]
            .convert("RGBA")
            .resize((220, 160), keep_ratio=True)
            .perspective(points)
        )
        return frame.copy().paste(screen.rotate(9, expand=True), (161, 117), below=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "play_game",
    play_game,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=[default_text],
    keywords=["玩游戏"],
    date_created=datetime(2022, 1, 4),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime

import numpy as np
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif


def rotate_3d(images: list[BuildImage], texts, args):
    tmp_img = images[0].convert("RGBA")
    fov = 45 * np.pi / 180
    z = np.sqrt(tmp_img.width**2 + tmp_img.height**2) / 2 / np.tan(fov / 2)
    w = round(tmp_img.width * 1.2)
    h = round(tmp_img.height * 1.5)
    bg = BuildImage.new("RGBA", (w, h))

    def rotate_y(img: BuildImage, theta: float) -> BuildImage:
        mat = np.array(
            [
                [np.cos(theta), 0, np.sin(theta)],
                [0, 1, 0],
                [-np.sin(theta), 0, np.cos(theta)],
            ]
        )
        pc = np.array([img.width / 2, img.height / 2, 0])
        orgs = [
            np.array([0, 0, 0]),
            np.array([img.width, 0, 0]),
            np.array([img.width, img.height, 0]),
            np.array([0, img.height, 0]),
        ]
        dsts = []
        for i in range(4):
            dst = mat.dot(orgs[i] - pc)
            dsts.append(
                [
                    dst[0] * z / (z - dst[2]) + pc[0],
                    dst[1] * z / (z - dst[2]) + pc[1],
                ]
            )
        min_x = min(dsts, key=lambda x: x[0])[0]
        min_y = min(dsts, key=lambda x: x[1])[1]
        for i in range(4):
            dsts[i][0] -= min_x
            dsts[i][1] -= min_y
        return img.perspective(tuple(dsts))

    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            img = imgs[0].convert("RGBA")
            frame = bg.copy()
            rotated = rotate_y(img, i * 12 * np.pi / 180)
            frame.paste(
                rotated,
                (
                    round((bg.width - rotated.width) / 2),
                    round((bg.height - rotated.height) / 2),
                ),
                alpha=True,
            )
            return frame

        return make

    return make_gif_or_combined_gif(
        images, maker, 30, 0.07, FrameAlignPolicy.extend_loop
    )


add_meme(
    "rotate_3d",
    rotate_3d,
    min_images=1,
    max_images=1,
    keywords=["三维旋转"],
    date_created=datetime(2024, 4, 30),
    date_modified=datetime(2024, 4, 30),
)



from dataclasses import dataclass
from datetime import datetime
from pathlib import Path

from arclet.alconna import store_true
from pil_utils import BuildImage
from pydantic import Field

from meme_generator import (
    MemeArgsModel,
    MemeArgsType,
    ParserOption,
    add_meme,
)

IMG_DIR = Path(__file__).parent / "images"
CIRCLE_PATH = IMG_DIR / "circle.png"
PERSON_PATH = IMG_DIR / "person.png"


@dataclass
class PicInfo:
    frame_path: Path
    avatar_size: tuple[int, int]
    avatar_rotate: int
    avatar_left_center: tuple[int, int]  # top right 直接算镜像


CIRCLE_INFO = PicInfo(CIRCLE_PATH, (554, 442), 26, (153, 341))
PERSON_INFO = PicInfo(PERSON_PATH, (434, 467), 26, (174, 378))


HELP_PERSON = "是否使用爷爷头轮廓"


class Model(MemeArgsModel):
    person: bool = Field(False, description=HELP_PERSON)


args_type = MemeArgsType(
    args_model=Model,
    args_examples=[Model(person=False), Model(person=True)],
    parser_options=[
        ParserOption(
            names=["--person", "爷"],
            default=False,
            action=store_true,
            help_text=HELP_PERSON,
        )
    ],
)


def clown(images: list[BuildImage], texts, args: Model):
    info = PERSON_INFO if args.person else CIRCLE_INFO
    avatar = images[0].convert("RGBA").resize(info.avatar_size, keep_ratio=True)
    frame = BuildImage.open(info.frame_path).convert("RGBA")

    img_size = frame.size
    bg = BuildImage.new("RGBA", img_size, (255, 255, 255))  # white bg

    left_part = avatar.crop(
        (0, 0, avatar.width // 2, avatar.height),
    ).rotate(info.avatar_rotate, expand=True)
    right_part = avatar.crop(
        (avatar.width // 2, 0, avatar.width, avatar.height)
    ).rotate(-info.avatar_rotate, expand=True)

    img_w = bg.width
    left_center_x, center_y = info.avatar_left_center  # 左半边中心 x, 中心 y
    left_top_x = left_center_x - left_part.width // 2  # 左半边左上 x
    top_y = center_y - left_part.height // 2  # 左上 y
    right_top_x = img_w - left_top_x - right_part.width  # 右半边左上 x

    bg.paste(left_part, (left_top_x, top_y), alpha=True)
    bg.paste(right_part, (right_top_x, top_y), alpha=True)
    bg.paste(frame, alpha=True)

    return bg.save_png()


add_meme(
    "clown",
    clown,
    min_images=1,
    max_images=1,
    args_type=args_type,
    keywords=["小丑"],
    date_created=datetime(2023, 10, 14),
    date_modified=datetime(2023, 10, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def prpr(images: list[BuildImage], texts, args):
    frame = BuildImage.open(img_dir / "0.png")

    def make(imgs: list[BuildImage]) -> BuildImage:
        points = ((0, 19), (236, 0), (287, 264), (66, 351))
        screen = (
            imgs[0]
            .convert("RGBA")
            .resize((330, 330), keep_ratio=True)
            .perspective(points)
        )
        return frame.copy().paste(screen, (56, 284), below=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "prpr",
    prpr,
    min_images=1,
    max_images=1,
    keywords=["舔", "舔屏", "prpr"],
    date_created=datetime(2022, 3, 5),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from typing import Literal

from arclet.alconna import store_value
from pil_utils import BuildImage
from pydantic import Field

from meme_generator import (
    CommandShortcut,
    MemeArgsModel,
    MemeArgsType,
    ParserArg,
    ParserOption,
    add_meme,
)
from meme_generator.utils import (
    FrameAlignPolicy,
    Maker,
    make_gif_or_combined_gif,
    make_jpg_or_gif,
)

help_text = "生成模式，包含 normal、loop、circle"


class Model(MemeArgsModel):
    mode: Literal["normal", "loop", "circle"] = Field("normal", description=help_text)


args_type = MemeArgsType(
    args_model=Model,
    args_examples=[Model(mode="normal"), Model(mode="circle"), Model(mode="loop")],
    parser_options=[
        ParserOption(
            names=["--mode"],
            args=[ParserArg(name="mode", value="str")],
            help_text=help_text,
        ),
        ParserOption(
            names=["--circle", "套娃"],
            dest="mode",
            action=store_value("circle"),
            help_text="套娃模式",
        ),
        ParserOption(
            names=["--loop", "循环"],
            dest="mode",
            action=store_value("loop"),
            help_text="循环模式",
        ),
    ],
)


def always_normal(img: BuildImage):
    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0]
        img_big = img.convert("RGBA").resize_width(500)
        img_small = img.convert("RGBA").resize_width(100)
        h1 = img_big.height
        h2 = max(img_small.height, 80)
        frame = BuildImage.new("RGBA", (500, h1 + h2 + 10), "white")
        frame.paste(img_big, alpha=True).paste(
            img_small, (290, h1 + 5 + (h2 - img_small.height) // 2), alpha=True
        )
        frame.draw_text(
            (20, h1 + 5, 280, h1 + h2 + 5), "要我一直", halign="right", max_fontsize=60
        )
        frame.draw_text(
            (400, h1 + 5, 480, h1 + h2 + 5), "吗", halign="left", max_fontsize=60
        )
        return frame

    return make_jpg_or_gif([img], make)


def always_always(img: BuildImage, loop: bool = False):
    tmp_img = img.convert("RGBA").resize_width(500)
    img_h = tmp_img.height
    text_h = tmp_img.resize_width(100).height + tmp_img.resize_width(20).height + 10
    text_h = max(text_h, 80)
    frame_h = img_h + text_h
    text_frame = BuildImage.new("RGBA", (500, frame_h), "white")
    text_frame.draw_text(
        (0, img_h, 280, frame_h), "要我一直", halign="right", max_fontsize=60
    ).draw_text((400, img_h, 500, frame_h), "吗", halign="left", max_fontsize=60)

    frame_num = 20
    coeff = 5 ** (1 / frame_num)

    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            img = imgs[0].convert("RGBA").resize_width(500)
            base_frame = text_frame.copy().paste(img, alpha=True)
            frame = BuildImage.new("RGBA", base_frame.size, "white")
            r = coeff**i
            for _ in range(4):
                x = round(358 * (1 - r))
                y = round(frame_h * (1 - r))
                w = round(500 * r)
                h = round(frame_h * r)
                frame.paste(base_frame.resize((w, h)), (x, y))
                r /= 5
            return frame

        return make

    if not loop:
        return make_jpg_or_gif([img], maker(0))

    return make_gif_or_combined_gif(
        [img], maker, frame_num, 0.1, FrameAlignPolicy.extend_loop
    )


def always(images: list[BuildImage], texts, args: Model):
    img = images[0]
    mode = args.mode

    if mode == "normal":
        return always_normal(img)
    elif mode == "circle":
        return always_always(img, loop=False)
    else:
        return always_always(img, loop=True)


add_meme(
    "always",
    always,
    min_images=1,
    max_images=1,
    args_type=args_type,
    keywords=["一直"],
    shortcuts=[CommandShortcut(key="一直一直", args=["--loop"])],
    date_created=datetime(2021, 12, 2),
    date_modified=datetime(2024, 8, 9),
)



import random
from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage
from pydantic import Field

from meme_generator import (
    CommandShortcut,
    MemeArgsModel,
    MemeArgsType,
    ParserArg,
    ParserOption,
    add_meme,
)
from meme_generator.exception import MemeFeedback
from meme_generator.tags import MemeTags
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif

img_dir = Path(__file__).parent / "images"

help_text = "角色编号：1、八重神子，2、胡桃，3、妮露，4、可莉，5、刻晴，6、钟离"


class Model(MemeArgsModel):
    character: int = Field(0, description=help_text)


args_type = MemeArgsType(
    args_model=Model,
    args_examples=[Model(character=i) for i in range(1, 7)],
    parser_options=[
        ParserOption(
            names=["-c", "--character", "角色"],
            args=[ParserArg(name="character", value="int")],
            help_text=help_text,
        ),
    ],
)


def genshin_eat(images: list[BuildImage], texts, args: Model):
    names = ["yae_miko", "hutao", "nilou", "klee", "keqing", "zhongli"]
    if args.character == 0:
        name = random.choice(names)
    elif args.character not in range(1, 7):
        raise MemeFeedback("角色编号错误，请选择1-6")
    else:
        name = names[args.character - 1]

    position_list = [(106, 245), (115, 224), (116, 205), (115, 198), (120, 217)]

    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            chara = BuildImage.open(img_dir / name / f"{i:02d}.png")
            if i in range(4, 9):
                food = (
                    imgs[0].convert("RGBA").circle().resize((44, 44), keep_ratio=True)
                )
                if i == 8:
                    food = food.resize((44, 33))
                chara.paste(food, position_list[i - 4], alpha=True)
            return chara

        return make

    return make_gif_or_combined_gif(
        images, maker, 16, 0.08, FrameAlignPolicy.extend_loop
    )


add_meme(
    "genshin_eat",
    genshin_eat,
    min_images=1,
    max_images=1,
    args_type=args_type,
    keywords=["原神吃"],
    shortcuts=[
        CommandShortcut(
            key=r"(?:八重神子|神子|八重)吃",
            args=["--character", "1"],
            humanized="八重神子吃",
        ),
        CommandShortcut(key="胡桃吃", args=["--character", "2"]),
        CommandShortcut(key="妮露吃", args=["--character", "3"]),
        CommandShortcut(key="可莉吃", args=["--character", "4"]),
        CommandShortcut(key="刻晴吃", args=["--character", "5"]),
        CommandShortcut(key="钟离吃", args=["--character", "6"]),
    ],
    tags=MemeTags.yae_miko
    | MemeTags.hutao
    | MemeTags.nilou
    | MemeTags.klee
    | MemeTags.keqing
    | MemeTags.zhongli,
    date_created=datetime(2024, 8, 6),
    date_modified=datetime(2024, 8, 10),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif

img_dir = Path(__file__).parent / "images"


def capoo_strike(images: list[BuildImage], texts, args):
    params = (
        (((0, 4), (153, 0), (138, 105), (0, 157)), (28, 47)),
        (((1, 13), (151, 0), (130, 104), (0, 156)), (28, 48)),
        (((9, 10), (156, 0), (152, 108), (0, 155)), (18, 51)),
        (((0, 21), (150, 0), (146, 115), (7, 145)), (17, 53)),
        (((0, 19), (156, 0), (199, 109), (31, 145)), (2, 62)),
        (((0, 28), (156, 0), (171, 115), (12, 154)), (16, 58)),
        (((0, 25), (157, 0), (169, 113), (13, 147)), (18, 63)),
    )

    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            img = imgs[0].convert("RGBA").resize((200, 160), keep_ratio=True)
            points, pos = params[i]
            frame = BuildImage.open(img_dir / f"{i}.png")
            frame.paste(img.perspective(points), pos, below=True)
            return frame

        return make

    return make_gif_or_combined_gif(
        images, maker, 7, 0.05, FrameAlignPolicy.extend_loop
    )


add_meme(
    "capoo_strike",
    capoo_strike,
    min_images=1,
    max_images=1,
    keywords=["咖波撞", "咖波头槌"],
    tags=MemeTags.capoo,
    date_created=datetime(2023, 3, 28),
    date_modified=datetime(2023, 3, 28),
)



import random
from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage, Text2Image

from meme_generator import MemeArgsModel, add_meme
from meme_generator.exception import TextOrNameNotEnough, TextOverLength

img_dir = Path(__file__).parent / "images"


def always_like(images: list[BuildImage], texts: list[str], args: MemeArgsModel):
    names = [info.name for info in args.user_infos]

    if len(images) > len(texts) + len(names):
        raise TextOrNameNotEnough()
    texts = texts + names

    img = images[0].convert("RGBA")
    name = texts[0]
    text = "我永远喜欢" + name

    frame = BuildImage.open(img_dir / "0.png")
    frame.paste(
        img.resize((350, 400), keep_ratio=True, inside=True), (25, 35), alpha=True
    )
    try:
        frame.draw_text(
            (20, 470, frame.width - 20, 570),
            text,
            max_fontsize=70,
            min_fontsize=30,
            weight="bold",
        )
    except ValueError:
        raise TextOverLength(text)

    def random_color():
        return random.choice(
            ["red", "darkorange", "gold", "darkgreen", "blue", "cyan", "purple"]
        )

    if len(images) > 1:
        text_w = Text2Image.from_text(text, 70).width
        ratio = min((frame.width - 40) / text_w, 1)
        text_w *= ratio
        name_w = Text2Image.from_text(name, 70).width * ratio
        start_w = text_w - name_w + (frame.width - text_w) // 2
        frame.draw_line(
            (start_w, 525, start_w + name_w, 525), fill=random_color(), width=10
        )

    current_h = 400
    for i, (image, name) in enumerate(zip(images[1:], texts[1:]), start=1):
        img = image.convert("RGBA")
        frame.paste(
            img.resize((350, 400), keep_ratio=True, inside=True),
            (10 + random.randint(0, 50), 20 + random.randint(0, 70)),
            alpha=True,
        )
        try:
            frame.draw_text(
                (400, current_h, frame.width - 20, current_h + 80),
                name,
                max_fontsize=70,
                min_fontsize=30,
                weight="bold",
            )
        except ValueError:
            raise TextOverLength(text)

        if len(images) > i + 1:
            name_w = min(Text2Image.from_text(name, 70).width, 380)
            start_w = 400 + (410 - name_w) // 2
            line_h = current_h + 40
            frame.draw_line(
                (start_w, line_h, start_w + name_w, line_h),
                fill=random_color(),
                width=10,
            )
        current_h -= 70
    return frame.save_jpg()


add_meme(
    "always_like",
    always_like,
    min_images=1,
    max_images=6,
    min_texts=0,
    max_texts=6,
    keywords=["我永远喜欢"],
    date_created=datetime(2022, 3, 14),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Transpose
from pil_utils import BuildImage, Text2Image

from meme_generator import add_meme

img_dir = Path(__file__).parent / "images"


def youtube(images, texts: list[str], args):
    left_img = Text2Image.from_text(texts[0], fontsize=200, fill="black").to_image(
        bg_color="white", padding=(30, 20)
    )

    right_img = Text2Image.from_text(
        texts[1], fontsize=200, fill="white", weight="bold"
    ).to_image(bg_color=(230, 33, 23), padding=(50, 20))
    right_img = BuildImage(right_img).resize_canvas(
        (max(right_img.width, 400), right_img.height), bg_color=(230, 33, 23)
    )
    right_img = right_img.circle_corner(right_img.height // 2)

    frame = BuildImage.new(
        "RGBA",
        (left_img.width + right_img.width, max(left_img.height, right_img.height)),
        "white",
    )
    frame.paste(left_img, (0, frame.height - left_img.height))
    frame = frame.resize_canvas(
        (frame.width + 100, frame.height + 100), bg_color="white"
    )

    corner = BuildImage.open(img_dir / "corner.png")
    ratio = right_img.height / 2 / corner.height
    corner = corner.resize((int(corner.width * ratio), int(corner.height * ratio)))
    x0 = left_img.width + 50
    y0 = frame.height - right_img.height - 50
    x1 = frame.width - corner.width - 50
    y1 = frame.height - corner.height - 50
    frame.paste(corner, (x0, y0 - 1), alpha=True).paste(
        corner.transpose(Transpose.FLIP_TOP_BOTTOM), (x0, y1 + 1), alpha=True
    ).paste(
        corner.transpose(Transpose.FLIP_LEFT_RIGHT), (x1, y0 - 1), alpha=True
    ).paste(
        corner.transpose(Transpose.FLIP_TOP_BOTTOM).transpose(
            Transpose.FLIP_LEFT_RIGHT
        ),
        (x1, y1 + 1),
        alpha=True,
    ).paste(right_img, (x0, y0), alpha=True)
    return frame.save_jpg()


add_meme(
    "youtube",
    youtube,
    min_texts=2,
    max_texts=2,
    default_texts=["Porn", "Hub"],
    keywords=["yt", "youtube"],
    date_created=datetime(2022, 10, 27),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def punch(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square().resize((260, 260))
    frames: list[IMG] = []
    # fmt: off
    locs = [
        (-50, 20), (-40, 10), (-30, 0), (-20, -10), (-10, -10), (0, 0),
        (10, 10), (20, 20), (10, 10), (0, 0), (-10, -10), (10, 0), (-30, 10)
    ]
    # fmt: on
    for i in range(13):
        fist = BuildImage.open(img_dir / f"{i}.png")
        frame = BuildImage.new("RGBA", fist.size, "white")
        x, y = locs[i]
        frame.paste(img, (x, y - 15), alpha=True).paste(fist, alpha=True)
        frames.append(frame.image)
    return save_gif(frames, 0.03)


add_meme(
    "punch",
    punch,
    min_images=1,
    max_images=1,
    keywords=["打拳"],
    date_created=datetime(2022, 3, 18),
    date_modified=datetime(2023, 2, 14),
)



import random
from datetime import datetime

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif


def shock(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square().resize((300, 300))
    frames: list[IMG] = []
    for i in range(30):
        frames.append(
            img.motion_blur(random.randint(-90, 90), random.randint(0, 50))
            .rotate(random.randint(-20, 20))
            .image
        )
    return save_gif(frames, 0.01)


add_meme(
    "shock",
    shock,
    min_images=1,
    max_images=1,
    keywords=["震惊"],
    date_created=datetime(2022, 3, 12),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.tags import MemeTags
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"

default_text = "阿尼亚喜欢这个"


def anya_suki(images: list[BuildImage], texts: list[str], args):
    text = texts[0] if texts else default_text
    frame = BuildImage.open(img_dir / "0.png")
    try:
        frame.draw_text(
            (5, frame.height - 60, frame.width - 5, frame.height - 10),
            text,
            max_fontsize=40,
            fill="white",
            stroke_fill="black",
            stroke_ratio=0.06,
        )
    except ValueError:
        raise TextOverLength(text)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((305, 235), keep_ratio=True)
        return frame.copy().paste(img, (106, 72), below=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "anya_suki",
    anya_suki,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=[default_text],
    keywords=["阿尼亚喜欢"],
    tags=MemeTags.anya,
    date_created=datetime(2022, 5, 10),
    date_modified=datetime(2023, 2, 14),
)



import importlib
import importlib.util
import pkgutil
from datetime import datetime
from pathlib import Path
from typing import Optional, Union

from .config import meme_config
from .exception import NoSuchMeme
from .log import logger
from .meme import CommandShortcut, Meme, MemeArgsType, MemeFunction, MemeParamsType

_memes: dict[str, Meme] = {}


def path_to_module_name(path: Path) -> str:
    rel_path = path.resolve().relative_to(Path.cwd().resolve())
    if rel_path.stem == "__init__":
        return ".".join(rel_path.parts[:-1])
    else:
        return ".".join(rel_path.parts[:-1] + (rel_path.stem,))


def load_meme(module_path: Union[str, Path]):
    module_name = (
        path_to_module_name(module_path)
        if isinstance(module_path, Path)
        else module_path
    )
    try:
        importlib.import_module(module_name)
    except Exception as e:
        logger.opt(colors=True, exception=e).error(f"Failed to import {module_path}!")


def load_memes(dir_path: Union[str, Path]):
    if isinstance(dir_path, Path):
        dir_path = str(dir_path.resolve())

    for module_info in pkgutil.iter_modules([dir_path]):
        if module_info.name.startswith("_"):
            continue
        if not (
            module_spec := module_info.module_finder.find_spec(module_info.name, None)
        ):
            continue
        if not (module_path := module_spec.origin):
            continue
        if not (module_loader := module_spec.loader):
            continue
        try:
            module = importlib.util.module_from_spec(module_spec)
            module_loader.exec_module(module)
        except Exception as e:
            logger.opt(colors=True, exception=e).error(
                f"Failed to import {module_path}!"
            )


def add_meme(
    key: str,
    function: MemeFunction,
    *,
    min_images: int = 0,
    max_images: int = 0,
    min_texts: int = 0,
    max_texts: int = 0,
    default_texts: list[str] = [],
    args_type: Optional[MemeArgsType] = None,
    keywords: list[str] = [],
    shortcuts: list[CommandShortcut] = [],
    tags: set[str] = set(),
    date_created: datetime = datetime(2021, 5, 4),
    date_modified: datetime = datetime.now(),
):
    if key in _memes:
        logger.warning(f'Meme with key "{key}" already exists!')
        return

    if key in meme_config.meme.meme_disabled_list:
        logger.warning(f'The key "{key}" is in the disabled list!')
        return

    meme = Meme(
        key,
        function,
        MemeParamsType(
            min_images, max_images, min_texts, max_texts, default_texts, args_type
        ),
        keywords=keywords,
        shortcuts=shortcuts,
        tags=tags,
        date_created=date_created,
        date_modified=date_modified,
    )

    _memes[key] = meme


def get_meme(key: str) -> Meme:
    if key not in _memes:
        raise NoSuchMeme(key)
    return _memes[key]


def get_memes() -> list[Meme]:
    return list(_memes.values())


def get_meme_keys() -> list[str]:
    return list(_memes.keys())



from datetime import datetime
from pathlib import Path

from PIL import ImageFilter
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def luoyonghao_say(images, texts: list[str], args):
    text = texts[0]
    frame = BuildImage.open(img_dir / "0.jpg")
    text_frame = BuildImage.new("RGBA", (365, 120))
    try:
        text_frame.draw_text(
            (40, 10, 325, 110),
            text,
            allow_wrap=True,
            max_fontsize=50,
            min_fontsize=10,
            valign="top",
        )
    except ValueError:
        raise TextOverLength(text)
    text_frame = text_frame.perspective(
        ((52, 10), (391, 0), (364, 110), (0, 120))
    ).filter(ImageFilter.GaussianBlur(radius=0.8))  # type: ignore
    frame.paste(text_frame, (48, 246), alpha=True)
    return frame.save_jpg()


add_meme(
    "luoyonghao_say",
    luoyonghao_say,
    min_texts=1,
    max_texts=1,
    default_texts=["又不是不能用"],
    keywords=["罗永浩说"],
    date_created=datetime(2023, 3, 28),
    date_modified=datetime(2023, 3, 28),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme

img_dir = Path(__file__).parent / "images"


def rip(images: list[BuildImage], texts, args):
    if len(images) >= 2:
        frame = BuildImage.open(img_dir / "1.png")
        self_img = images[0]
        user_img = images[1]
    else:
        frame = BuildImage.open(img_dir / "0.png")
        self_img = None
        user_img = images[0]

    user_img = user_img.convert("RGBA").square().resize((385, 385))
    if self_img:
        self_img = self_img.convert("RGBA").square().resize((230, 230))
        frame.paste(self_img, (408, 418), below=True)
    frame.paste(user_img.rotate(24, expand=True), (-5, 355), below=True)
    frame.paste(user_img.rotate(-11, expand=True), (649, 310), below=True)
    return frame.save_jpg()


add_meme(
    "rip",
    rip,
    min_images=1,
    max_images=2,
    keywords=["撕"],
    date_created=datetime(2021, 5, 5),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme

img_dir = Path(__file__).parent / "images"


def potato(images: list[BuildImage], texts, args):
    frame = BuildImage.open(img_dir / "0.png")
    img = images[0].convert("RGBA").square().resize((458, 458))
    frame.paste(img.rotate(-5), (531, 15), below=True)
    return frame.save_jpg()


add_meme(
    "potato",
    potato,
    min_images=1,
    max_images=1,
    keywords=["土豆"],
    date_created=datetime(2023, 1, 19),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"

default_text = "此处添加文字"


def father_work(images: list[BuildImage], texts: list[str], args):
    frame = BuildImage.open(img_dir / "0.png")
    text = texts[0] if texts else default_text
    try:
        frame.draw_text(
            (195, frame.height - 110, frame.width - 10, frame.height - 20),
            text,
            min_fontsize=10,
            max_fontsize=50,
            fill="black",
            allow_wrap=True,
            lines_align="center",
        )
    except ValueError:
        raise TextOverLength(text)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((230, 120), keep_ratio=True, inside=True)
        return frame.copy().paste(img, (252, 142), alpha=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "father_work",
    father_work,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=[default_text],
    keywords=["闭嘴", "我爸爸"],
    date_created=datetime(2024, 5, 12),
    date_modified=datetime(2024, 5, 16),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.tags import MemeTags

img_dir = Path(__file__).parent / "images"


def my_opinion(images, texts: list[str], args):
    text = texts[0]
    frame = BuildImage.open(img_dir / "0.png")
    try:
        frame.draw_text(
            (37, 660, 487, 1070),
            text,
            max_fontsize=500,
            min_fontsize=20,
            weight="bold",
        )
    except ValueError:
        raise TextOverLength(text)
    return frame.save_png()


add_meme(
    "my_opinion",
    my_opinion,
    min_texts=1,
    max_texts=1,
    keywords=["我的意见如下", "我的意见是"],
    tags=MemeTags.touhou,
    date_created=datetime(2024, 7, 14),
    date_modified=datetime(2024, 7, 14),
)



from datetime import datetime

import dateparser
from pil_utils import BuildImage, Text2Image
from pydantic import Field

from meme_generator import (
    MemeArgsModel,
    MemeArgsType,
    ParserArg,
    ParserOption,
    add_meme,
)
from meme_generator.exception import TextOverLength


class Model(MemeArgsModel):
    time: str = Field("", description="指定时间")
    name: str = Field("", description="指定名字")


args_type = MemeArgsType(
    args_model=Model,
    parser_options=[
        ParserOption(
            names=["-t", "--time"],
            args=[ParserArg(name="time", value="str")],
            help_text="指定时间",
        ),
        ParserOption(
            names=["-n", "--name"],
            args=[ParserArg(name="name", value="str")],
            help_text="指定名字",
        ),
    ],
)

default_text = "想玩原神"


def note_for_leave(images: list[BuildImage], texts: list[str], args: Model):
    time = datetime.now()
    if args.time and (parsed_time := dateparser.parse(args.time)):
        time = parsed_time
    name = args.name or (args.user_infos[-1].name if args.user_infos else "")
    text = texts[0] if texts else default_text
    img = (
        images[0]
        .convert("RGBA")
        .resize((450, 400), keep_ratio=True, inside=True, bg_color="white")
    )

    frame = BuildImage.new("RGBA", (800, 950), "white")
    frame.draw_text(
        (40, 20, 760, 180),
        text="请假条",
        weight="bold",
        max_fontsize=100,
        min_fontsize=80,
    )
    frame.draw_text((40, 200), "本人", fontsize=50)
    name_width = Text2Image.from_text(name, fontsize=50).width
    if (name_width) > 800:
        raise TextOverLength(name)
    name_width = min(450, max(150, name_width)) + 50
    frame.draw_text(
        (150, 200, 150 + name_width, 265), text=name, max_fontsize=50, min_fontsize=20
    )
    frame.draw_line((150, 260, 150 + name_width, 260), fill="black", width=4)
    frame.draw_text((160 + name_width, 200), "因", fontsize=50)
    try:
        frame.draw_text(
            (40, 300, 285, 700),
            text=text,
            max_fontsize=90,
            min_fontsize=40,
            allow_wrap=True,
            lines_align="center",
            fill="red",
        )
    except ValueError:
        raise TextOverLength(text)
    frame.paste(img, (300, 290))
    frame.draw_bbcode_text(
        (40, 700, 760, 800),
        text=f"于[u] {time.year} [/u]年[u] {time.month} [/u]月[u] {time.day} [/u]日请假一天,",
        max_fontsize=50,
        min_fontsize=30,
        halign="left",
    )
    frame.draw_text((40, 800), text="望领导批准！！！", fontsize=75, weight="bold")
    return frame.save_jpg()


add_meme(
    "note_for_leave",
    note_for_leave,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=[default_text],
    args_type=args_type,
    keywords=["请假条"],
    date_created=datetime(2023, 4, 27),
    date_modified=datetime(2023, 4, 27),
)



from typing import Optional


class MemeGeneratorException(Exception):
    status_code: int = 520

    def __init__(self, message: str):
        self.message = message

    def __str__(self) -> str:
        return self.__repr__()

    def __repr__(self) -> str:
        return f"表情制作出错！{self.message}"


class NoSuchMeme(MemeGeneratorException):
    status_code: int = 531

    def __init__(self, meme_key: str):
        self.meme_key = meme_key
        message = f"表情“{self.meme_key}”不存在"
        super().__init__(message)


class TextOverLength(MemeGeneratorException):
    status_code: int = 532

    def __init__(self, text: str):
        self.text = text
        repr = text if len(text) <= 10 else (text[:10] + "...")
        message = f"文本“{repr}”过长"
        super().__init__(message)


class OpenImageFailed(MemeGeneratorException):
    status_code: int = 533

    def __init__(self, error_message: str):
        self.error_message = error_message
        message = f"图片加载失败（{self.error_message}）"
        super().__init__(message)


class ParamsMismatch(MemeGeneratorException):
    status_code: int = 540


class ImageNumberMismatch(ParamsMismatch):
    status_code: int = 541

    def __init__(self, min_images: int = 0, max_images: int = 0):
        self.min_images = min_images
        self.max_images = max_images
        message = f"图片数量不符，图片数量应为 {min_images}" + (
            f" ~ {max_images}" if max_images > min_images else ""
        )
        super().__init__(message)


class TextNumberMismatch(ParamsMismatch):
    status_code: int = 542

    def __init__(self, min_texts: int = 0, max_texts: int = 0):
        self.min_texts = min_texts
        self.max_texts = max_texts
        message = f"文本数量不符，文本数量应为 {min_texts}" + (
            f" ~ {max_texts}" if max_texts > min_texts else ""
        )
        super().__init__(message)


class TextOrNameNotEnough(ParamsMismatch):
    status_code: int = 543

    def __init__(self, message: Optional[str] = None):
        message = message or "文本或用户名数量不足"
        super().__init__(message)


class ArgMismatch(ParamsMismatch):
    status_code: int = 550


class ArgParserMismatch(ArgMismatch):
    status_code: int = 551

    def __init__(self, error_message: str):
        self.error_message = error_message
        message = f"参数解析失败（{self.error_message}）"
        super().__init__(message)


class ArgModelMismatch(ArgMismatch):
    status_code: int = 552

    def __init__(self, error_message: str):
        self.error_message = error_message
        message = f"参数模型验证失败（{self.error_message}）"
        super().__init__(message)


class MemeFeedback(MemeGeneratorException):
    status_code: int = 560



from datetime import datetime

from pil_utils import BuildImage

from meme_generator import add_meme


def trance(images: list[BuildImage], texts, args):
    img = images[0]
    width, height = img.size
    height1 = int(1.1 * height)
    frame = BuildImage.new("RGB", (width, height1), "white")
    frame.paste(img, (0, int(height * 0.1)))
    img.image.putalpha(3)
    for i in range(int(height * 0.1), 0, -1):
        frame.paste(img, (0, i), alpha=True)
    for i in range(int(height * 0.1), int(height * 0.1 * 2)):
        frame.paste(img, (0, i), alpha=True)
    frame = frame.crop((0, int(0.1 * height), width, height1))
    return frame.save_jpg()


add_meme(
    "trance",
    trance,
    min_images=1,
    max_images=1,
    keywords=["恍惚"],
    date_created=datetime(2022, 12, 11),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime

from PIL import Image, ImageDraw
from pil_utils import BuildImage
from pil_utils.fonts import Font

from meme_generator import add_meme
from meme_generator.utils import make_jpg_or_gif


def charpic(images: list[BuildImage], texts, args):
    str_map = "@@$$&B88QMMGW##EE93SPPDOOU**==()+^,\"--''.  "
    num = len(str_map)
    font = Font.find("Consolas").load_font(15)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize_width(150).convert("L")
        img = img.resize((img.width, img.height // 2))
        lines = []
        for y in range(img.height):
            line = ""
            for x in range(img.width):
                gray = img.image.getpixel((x, y))
                line += str_map[int(num * gray / 256)] if gray != 0 else " "  # type: ignore
            lines.append(line)
        text = "\n".join(lines)
        text_img = Image.new("RGB", (2000, 2000), "white")
        draw = ImageDraw.Draw(text_img)
        _, _, w, h = draw.multiline_textbbox((0, 0), text, font=font)
        draw.multiline_text((0, 0), text, font=font, fill="black")
        text_img = text_img.crop((0, 0, w, h))
        return BuildImage(text_img)

    return make_jpg_or_gif(images, make)


add_meme(
    "charpic",
    charpic,
    min_images=1,
    max_images=1,
    keywords=["字符画"],
    date_created=datetime(2022, 7, 21),
    date_modified=datetime(2023, 10, 2),
)



import math
from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags

img_dir = Path(__file__).parent / "images"


def lost_dog(images: list[BuildImage], texts, args):
    k = 2
    w_ = 663 * k
    w = 540 * k
    h = 540 * k
    r = 466 * k
    img = images[0].convert("RGBA").resize((w_, h), keep_ratio=True)
    img_new = BuildImage.new("RGBA", (w, h))
    for x in range(w):
        for y in range(h):
            theta = math.asin(abs(x - w / 2) / r)
            x_ = round(w_ / 2 + (x - w / 2) / math.cos(theta))
            y_ = round(h / 2 + (y - h / 2) / math.cos(theta))
            if 0 <= x_ < w_ and 0 <= y_ < h:
                img_new.image.putpixel((x, y), img.image.getpixel((x_, y_)))  # type: ignore
    img_new = img_new.resize((w // k, h // k))
    frame = BuildImage.open(img_dir / "0.png")
    frame.paste(img_new, (295, 165), below=True)
    return frame.save_jpg()


add_meme(
    "lost_dog",
    lost_dog,
    min_images=1,
    max_images=1,
    keywords=["寻狗启事"],
    tags=MemeTags.ayaka,
    date_created=datetime(2024, 1, 19),
    date_modified=datetime(2024, 1, 20),
)



from datetime import datetime

from pil_utils import BuildImage, Text2Image

from meme_generator import add_meme
from meme_generator.utils import make_jpg_or_gif


def universal(images: list[BuildImage], texts: list[str], args):
    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize_width(500)
        frames: list[BuildImage] = [img]
        for text in texts:
            text_img = BuildImage(
                Text2Image.from_bbcode_text(text, fontsize=45, align="center")
                .wrap(480)
                .to_image()
            )
            frames.append(text_img.resize_canvas((500, text_img.height)))

        frame = BuildImage.new(
            "RGBA", (500, sum(f.height for f in frames) + 10), "white"
        )
        current_h = 0
        for f in frames:
            frame.paste(f, (0, current_h), alpha=True)
            current_h += f.height
        return frame

    return make_jpg_or_gif(images, make)


add_meme(
    "universal",
    universal,
    min_images=1,
    max_images=1,
    min_texts=1,
    max_texts=10,
    default_texts=["在此处添加文字"],
    keywords=["万能表情", "空白表情"],
    date_created=datetime(2022, 4, 20),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from arclet.alconna import store_true
from pil_utils import BuildImage
from pydantic import Field

from meme_generator import MemeArgsModel, MemeArgsType, ParserOption, add_meme
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"

help_text = "是否将图片变为黑白"


class Model(MemeArgsModel):
    black: bool = Field(False, description=help_text)


args_type = MemeArgsType(
    args_model=Model,
    args_examples=[Model(black=False), Model(black=True)],
    parser_options=[
        ParserOption(
            names=["--black", "黑白"],
            default=False,
            action=store_true,
            help_text=help_text,
        ),
    ],
)


def mourning(images: list[BuildImage], texts, args: Model):
    frame = BuildImage.open(img_dir / "0.png")

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0]
        img = img.convert("L") if args.black else img.convert("RGBA")
        img = img.resize((635, 725), keep_ratio=True)
        return frame.copy().paste(img, (645, 145), below=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "mourning",
    mourning,
    min_images=1,
    max_images=1,
    args_type=args_type,
    keywords=["上香"],
    date_created=datetime(2023, 7, 29),
    date_modified=datetime(2023, 7, 29),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import CommandShortcut, add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def genshin_start(images: list[BuildImage], texts: list[str], args):
    frame = BuildImage.open(img_dir / "0.png")
    if texts:
        text = texts[0]
        try:
            frame.draw_text(
                (100, frame.height - 150, frame.width - 100, frame.height),
                text,
                max_fontsize=100,
                min_fontsize=70,
                fill="white",
                stroke_fill="black",
                stroke_ratio=0.05,
            )
        except ValueError:
            raise TextOverLength(text)

    def make(imgs: list[BuildImage]) -> BuildImage:
        points = ((0, 116), (585, 0), (584, 319), (43, 385))
        screen = (
            imgs[0]
            .convert("RGBA")
            .resize((600, 330), keep_ratio=True)
            .perspective(points)
        )
        return frame.copy().paste(screen, (412, 121), below=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "genshin_start",
    genshin_start,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=["原神，启动！"],
    keywords=["原神启动"],
    shortcuts=[
        CommandShortcut(
            key=r"(?P<text>\S+启动[!！]?)",
            args=["{text}"],
            humanized="xx启动",
        )
    ],
    date_created=datetime(2023, 7, 1),
    date_modified=datetime(2023, 7, 1),
)



from datetime import datetime

from pil_utils import BuildImage, Text2Image

from meme_generator import add_meme


def pornhub(images, texts: list[str], args):
    left_img = Text2Image.from_text(texts[0], fontsize=200, fill="white").to_image(
        bg_color="black", padding=(20, 10)
    )

    right_img = Text2Image.from_text(
        texts[1], fontsize=200, fill="black", weight="bold"
    ).to_image(bg_color=(247, 152, 23), padding=(20, 10))
    right_img = BuildImage(right_img).circle_corner(20)

    frame = BuildImage.new(
        "RGBA",
        (left_img.width + right_img.width, max(left_img.height, right_img.height)),
        "black",
    )
    frame.paste(left_img, (0, frame.height - left_img.height)).paste(
        right_img, (left_img.width, frame.height - right_img.height), alpha=True
    )
    frame = frame.resize_canvas(
        (frame.width + 100, frame.height + 100), bg_color="black"
    )
    return frame.save_jpg()


add_meme(
    "pornhub",
    pornhub,
    min_texts=2,
    max_texts=2,
    default_texts=["You", "Tube"],
    keywords=["ph", "pornhub"],
    date_created=datetime(2022, 10, 27),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def garbage(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square().resize((79, 79))
    # fmt: off
    locs = (
        [] + [(39, 40)] * 3 + [(39, 30)] * 2 + [(39, 32)] * 10
        + [(39, 30), (39, 27), (39, 32), (37, 49), (37, 64),
           (37, 67), (37, 67), (39, 69), (37, 70), (37, 70)]
    )
    # fmt: on
    frames: list[IMG] = []
    for i in range(25):
        frame = BuildImage.open(img_dir / f"{i}.png")
        frame.paste(img, locs[i], below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.04)


add_meme(
    "garbage",
    garbage,
    min_images=1,
    max_images=1,
    keywords=["垃圾", "垃圾桶"],
    date_created=datetime(2022, 4, 14),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage, Text2Image

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def hold_grudge(images, texts: list[str], args):
    date = datetime.today().strftime("%Y{}%m{}%d{}").format("年", "月", "日")
    text = f"{date} 晴\n{texts[0]}\n这个仇我先记下了"
    text2image = Text2Image.from_text(text, 45, fill="black", spacing=10).wrap(440)
    if len(text2image.lines) > 10:
        raise TextOverLength(texts[0])
    text_img = text2image.to_image()

    frame = BuildImage.open(img_dir / "0.png")
    bg = BuildImage.new(
        "RGB", (frame.width, frame.height + text_img.height + 20), "white"
    )
    bg.paste(frame).paste(text_img, (30, frame.height + 5), alpha=True)
    return bg.save_jpg()


add_meme(
    "hold_grudge",
    hold_grudge,
    min_texts=1,
    max_texts=1,
    default_texts=["群友不发涩图"],
    keywords=["记仇"],
    date_created=datetime(2021, 12, 15),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import MemeArgsModel, add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def you_should_call(images: list[BuildImage], texts: list[str], args: MemeArgsModel):
    frame = BuildImage.open(img_dir / "0.png")

    ta = "她"
    name = ta
    if texts:
        name = texts[0]
    elif args.user_infos:
        info = args.user_infos[0]
        ta = "他" if info.gender == "male" else "她"
        name = info.name or ta

    text = f"这种情况你应该致电{name}"
    try:
        frame.draw_text(
            (50, 20, 700, 130),
            text,
            max_fontsize=70,
            min_fontsize=20,
            valign="bottom",
        )
    except ValueError:
        raise TextOverLength(name)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").circle().resize((300, 300))
        return frame.copy().paste(img, (400, 190), alpha=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "you_should_call",
    you_should_call,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    keywords=["致电", "你应该致电"],
    date_created=datetime(2024, 7, 26),
    date_modified=datetime(2024, 7, 26),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags

img_dir = Path(__file__).parent / "images"


def why_at_me(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").resize((265, 265), keep_ratio=True)
    frame = BuildImage.open(img_dir / "0.png")
    frame.paste(img.rotate(19), (42, 13), below=True)
    return frame.save_jpg()


add_meme(
    "why_at_me",
    why_at_me,
    min_images=1,
    max_images=1,
    keywords=["为什么@我"],
    tags=MemeTags.touhou,
    date_created=datetime(2022, 4, 14),
    date_modified=datetime(2023, 5, 3),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif

img_dir = Path(__file__).parent / "images"


def vibrate(images: list[BuildImage], texts, args):
    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            img = imgs[0].convert("RGBA")
            w = img.size[0]
            y = img.size[1]
            locs = [
                (0, 0),
                (w // 25, y // 25),
                (w // 50, y // 50),
                (0, y // 25),
                (w // 25, 0),
            ]
            frame = BuildImage.new("RGBA", (w + w // 25, y + y // 25), "white")
            frame.paste(img, locs[i], alpha=True)
            return frame

        return make

    return make_gif_or_combined_gif(
        images, maker, 5, 0.05, FrameAlignPolicy.extend_loop
    )


add_meme(
    "vibrate",
    vibrate,
    min_images=1,
    max_images=1,
    keywords=["震动"],
    date_created=datetime(2023, 8, 28),
    date_modified=datetime(2023, 8, 28),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def throw_gif(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").circle()
    locs = [
        [(32, 32, 108, 36)],
        [(32, 32, 122, 36)],
        [],
        [(123, 123, 19, 129)],
        [(185, 185, -50, 200), (33, 33, 289, 70)],
        [(32, 32, 280, 73)],
        [(35, 35, 259, 31)],
        [(175, 175, -50, 220)],
    ]
    frames: list[IMG] = []
    for i in range(8):
        frame = BuildImage.open(img_dir / f"{i}.png")
        for w, h, x, y in locs[i]:
            frame.paste(img.resize((w, h)), (x, y), alpha=True)
        frames.append(frame.image)
    return save_gif(frames, 0.1)


add_meme(
    "throw_gif",
    throw_gif,
    min_images=1,
    max_images=1,
    keywords=["抛", "掷"],
    date_created=datetime(2022, 3, 9),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif

img_dir = Path(__file__).parent / "images"


def pepe_raise(images: list[BuildImage], texts, args):
    # fmt: off
    left_locs = [
        (107, 30), (107, 30), (95, 45),
        (80, 160), (80, 160), (70, 98),
    ]
    right_locs = [
        (320, 145), (320, 145), (330, 130),
        (300, 50), (300, 50), (323, 80),
    ]
    # fmt: on
    frame_num = 6

    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            frame = BuildImage.open(img_dir / f"{i}.png")
            left_img = imgs[0].convert("RGBA").circle().resize((100, 100))
            right_img = imgs[1].convert("RGBA").circle().resize((100, 100))
            frame.paste(left_img, left_locs[i], alpha=True)
            frame.paste(right_img, right_locs[i], alpha=True)
            return frame

        return make

    return make_gif_or_combined_gif(
        images, maker, frame_num, 0.06, FrameAlignPolicy.extend_loop
    )


add_meme(
    "pepe_raise",
    pepe_raise,
    min_images=2,
    max_images=2,
    keywords=["佩佩举"],
    tags=MemeTags.arknights,
    date_created=datetime(2024, 8, 18),
    date_modified=datetime(2024, 8, 18),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme

img_dir = Path(__file__).parent / "images"


def cover_face(images: list[BuildImage], texts, args):
    points = ((15, 15), (448, 0), (445, 456), (0, 465))
    img = images[0].convert("RGBA").square().resize((450, 450)).perspective(points)
    frame = BuildImage.open(img_dir / "0.png")
    frame.paste(img, (120, 150), below=True)
    return frame.save_jpg()


add_meme(
    "cover_face",
    cover_face,
    min_images=1,
    max_images=1,
    keywords=["捂脸"],
    date_created=datetime(2022, 3, 30),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def pass_the_buck(images: list[BuildImage], texts: list[str], args):
    img = images[0].convert("RGBA").square().resize((27, 27))
    frames: list[IMG] = []
    locs = [(2, 26), (10, 24), (15, 27), (17, 29), (10, 20), (2, 29), (3, 31), (1, 30)]
    for i in range(8):
        frame = BuildImage.open(img_dir / f"{i}.png")
        if texts:
            text = texts[0]
            try:
                frame.draw_text(
                    (0, 0, frame.width, 20), text, max_fontsize=20, min_fontsize=10
                )
            except ValueError:
                raise TextOverLength(text)
        frame.paste(img, locs[i], alpha=True)
        frames.append(frame.image)
    return save_gif(frames, 0.1)


add_meme(
    "pass_the_buck",
    pass_the_buck,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=["你写!"],
    keywords=["推锅", "甩锅"],
    date_created=datetime(2023, 3, 31),
    date_modified=datetime(2023, 4, 18),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def pat(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square()
    locs = [(11, 73, 106, 100), (8, 79, 112, 96)]
    img_frames: list[IMG] = []
    for i in range(10):
        frame = BuildImage.open(img_dir / f"{i}.png")
        x, y, w, h = locs[1] if i == 2 else locs[0]
        frame.paste(img.resize((w, h)), (x, y), below=True)
        img_frames.append(frame.image)
    # fmt: off
    seq = [0, 1, 2, 3, 1, 2, 3, 0, 1, 2, 3, 0, 0, 1, 2, 3, 0, 0, 0, 0, 4, 5, 5, 5, 6, 7, 8, 9]
    # fmt: on
    frames = [img_frames[n] for n in seq]
    return save_gif(frames, 0.085)


add_meme(
    "pat",
    pat,
    min_images=1,
    max_images=1,
    keywords=["拍"],
    date_created=datetime(2021, 12, 1),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def play(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square()
    # fmt: off
    locs = [
        (180, 60, 100, 100), (184, 75, 100, 100), (183, 98, 100, 100),
        (179, 118, 110, 100), (156, 194, 150, 48), (178, 136, 122, 69),
        (175, 66, 122, 85), (170, 42, 130, 96), (175, 34, 118, 95),
        (179, 35, 110, 93), (180, 54, 102, 93), (183, 58, 97, 92),
        (174, 35, 120, 94), (179, 35, 109, 93), (181, 54, 101, 92),
        (182, 59, 98, 92), (183, 71, 90, 96), (180, 131, 92, 101)
    ]
    # fmt: on
    raw_frames: list[BuildImage] = [
        BuildImage.open(img_dir / f"{i}.png") for i in range(38)
    ]
    img_frames: list[BuildImage] = []
    for i in range(len(locs)):
        frame = raw_frames[i]
        x, y, w, h = locs[i]
        frame.paste(img.resize((w, h)), (x, y), below=True)
        img_frames.append(frame)
    frames = (
        img_frames[0:12]
        + img_frames[0:12]
        + img_frames[0:8]
        + img_frames[12:18]
        + raw_frames[18:38]
    )
    frames = [frame.image for frame in frames]
    return save_gif(frames, 0.06)


add_meme(
    "play",
    play,
    min_images=1,
    max_images=1,
    keywords=["顶", "玩"],
    date_created=datetime(2021, 10, 9),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage
from pydantic import Field

from meme_generator import (
    MemeArgsModel,
    MemeArgsType,
    ParserArg,
    ParserOption,
    add_meme,
)
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"

help_pron = "人称代词，默认为“我”"
help_name = "称呼，默认为“老婆”"


class Model(MemeArgsModel):
    pronoun: str = Field("我", description=help_pron)
    name: str = Field("老婆", description=help_name)


args_type = MemeArgsType(
    args_model=Model,
    parser_options=[
        ParserOption(
            names=["-p", "--pron"],
            args=[ParserArg(name="pronoun", value="str")],
            dest="pronoun",
            help_text=help_pron,
        ),
        ParserOption(
            names=["-n", "--name"],
            args=[ParserArg(name="name", value="str")],
            dest="name",
            help_text=help_name,
        ),
    ],
)


def my_wife(images: list[BuildImage], texts, args: Model):
    img = images[0].convert("RGBA").resize_width(400)
    img_w, img_h = img.size
    frame = BuildImage.new("RGBA", (650, img_h + 500), "white")
    frame.paste(img, (int(325 - img_w / 2), 105), alpha=True)

    pron = args.pronoun
    name = args.name
    try:
        text = f"如果你的{name}长这样"
        frame.draw_text(
            (27, 12, 27 + 596, 12 + 79),
            text,
            max_fontsize=70,
            min_fontsize=30,
            allow_wrap=True,
            lines_align="center",
            weight="bold",
        )
        text = f"那么这就不是你的{name}\n这是{pron}的{name}"
        frame.draw_text(
            (27, img_h + 120, 27 + 593, img_h + 120 + 135),
            text,
            max_fontsize=70,
            min_fontsize=30,
            allow_wrap=True,
            weight="bold",
        )
        text = f"滚去找你\n自己的{name}去"
        frame.draw_text(
            (27, img_h + 295, 27 + 374, img_h + 295 + 135),
            text,
            max_fontsize=70,
            min_fontsize=30,
            allow_wrap=True,
            lines_align="center",
            weight="bold",
        )
    except ValueError:
        raise TextOverLength(name)

    img_point = BuildImage.open(img_dir / "point.png").resize_width(200)
    frame.paste(img_point, (421, img_h + 270))

    return frame.save_jpg()


add_meme(
    "my_wife",
    my_wife,
    min_images=1,
    max_images=1,
    keywords=["我老婆", "这是我老婆"],
    args_type=args_type,
    date_created=datetime(2022, 7, 29),
    date_modified=datetime(2024, 8, 12),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def kick_ball(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square().resize((78, 78))
    # fmt: off
    locs = [
        (57, 136), (56, 117), (55, 99), (52, 113), (50, 126),
        (48, 139), (47, 112), (47, 85), (47, 57), (48, 97),
        (50, 136), (51, 176), (52, 169), (55, 181), (58, 153)
    ]
    # fmt: on
    frames: list[IMG] = []
    for i in range(15):
        frame = BuildImage.open(img_dir / f"{i}.png")
        frame.paste(img.rotate(-24 * i), locs[i], below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.1)


add_meme(
    "kick_ball",
    kick_ball,
    min_images=1,
    max_images=1,
    keywords=["踢球"],
    date_created=datetime(2022, 11, 29),
    date_modified=datetime(2023, 2, 14),
)



# https://github.com/nonebot/nonebot2/blob/master/nonebot/log.py
import logging
import sys
from typing import TYPE_CHECKING

import loguru

from .compat import model_dump

if TYPE_CHECKING:
    from loguru import Logger, Record

logger: "Logger" = loguru.logger


class LoguruHandler(logging.Handler):
    """logging 与 loguru 之间的桥梁，将 logging 的日志转发到 loguru。"""

    def emit(self, record: logging.LogRecord):
        try:
            level = logger.level(record.levelname).name
        except ValueError:
            level = record.levelno

        frame, depth = logging.currentframe(), 2
        while frame and frame.f_code.co_filename == logging.__file__:
            frame = frame.f_back
            depth += 1

        logger.opt(depth=depth, exception=record.exc_info).log(
            level, record.getMessage()
        )


# https://github.com/nonebot/nonebot2/blob/master/nonebot/drivers/fastapi.py#L182
LOGGING_CONFIG = {
    "version": 1,
    "disable_existing_loggers": False,
    "handlers": {
        "default": {
            "class": "meme_generator.log.LoguruHandler",
        },
    },
    "loggers": {
        "uvicorn.error": {"handlers": ["default"], "level": "INFO"},
        "uvicorn.access": {
            "handlers": ["default"],
            "level": "INFO",
        },
    },
}


def setup_logger():
    from .config import config_file_path, meme_config

    def default_filter(record: "Record"):
        """默认的日志过滤器，根据 `log_level` 配置改变日志等级。"""
        log_level = meme_config.log.log_level
        levelno = (
            logger.level(log_level).no if isinstance(log_level, str) else log_level
        )
        return record["level"].no >= levelno

    default_format: str = (
        "<g>{time:MM-DD HH:mm:ss}</g> "
        "[<lvl>{level}</lvl>] "
        "<c><u>{name}</u></c> | "
        # "<c>{function}:{line}</c>| "
        "{message}"
    )

    logger.remove()
    logger.add(
        sys.stdout,
        level=0,
        diagnose=False,
        filter=default_filter,
        format=default_format,
    )

    logger.opt(colors=True).info(
        f"Config file path: <y><d>{config_file_path.resolve()}</d></y>"
    )
    logger.opt(colors=True).debug(
        f"Loaded config: <y><d>{model_dump(meme_config)}</d></y>"
    )
